@@ -6,6 +6,7 @@ import (
 	"os"
 	"path/filepath"
 	"regexp"
+	"sort"
 	"strconv"
 	"strings"
 	"sync"
@@ -63,10 +64,14 @@ func NewCommitLoader(
 type GetCommitsOptions struct {
 	Limit                bool
 	FilterPath           string
+	FilterAuthor         string
 	IncludeRebaseCommits bool
 	RefName              string // e.g. "HEAD" or "my_branch"
+	RefForPushedStatus   string // the ref to use for determining pushed/unpushed status
 	// determines if we show the whole git graph i.e. pass the '--all' flag
 	All bool
+	// If non-empty, show divergence from this ref (left-right log)
+	RefToShowDivergenceFrom string
 }
 
 // GetCommits obtains the commits of the current branch
@@ -92,22 +97,26 @@ func (self *CommitLoader) GetCommits(opts GetCommitsOptions) ([]*models.Commit,
 		defer wg.Done()
 
 		logErr = self.getLogCmd(opts).RunAndProcessLines(func(line string) (bool, error) {
-			commit := self.extractCommitFromLine(line)
+			commit := self.extractCommitFromLine(line, opts.RefToShowDivergenceFrom != "")
 			commits = append(commits, commit)
 			return false, nil
 		})
 	})
 
 	var ancestor string
+	var remoteAncestor string
 	go utils.Safe(func() {
 		defer wg.Done()
 
 		ancestor = self.getMergeBase(opts.RefName)
+		if opts.RefToShowDivergenceFrom != "" {
+			remoteAncestor = self.getMergeBase(opts.RefToShowDivergenceFrom)
+		}
 	})
 
 	passedFirstPushedCommit := false
 	// I can get this before
-	firstPushedCommit, err := self.getFirstPushedCommit(opts.RefName)
+	firstPushedCommit, err := self.getFirstPushedCommit(opts.RefForPushedStatus)
 	if err != nil {
 		// must have no upstream branch so we'll consider everything as pushed
 		passedFirstPushedCommit = true
@@ -123,10 +132,12 @@ func (self *CommitLoader) GetCommits(opts GetCommitsOptions) ([]*models.Commit,
 		if commit.Sha == firstPushedCommit {
 			passedFirstPushedCommit = true
 		}
-		if passedFirstPushedCommit {
-			commit.Status = models.StatusPushed
-		} else {
-			commit.Status = models.StatusUnpushed
+		if commit.Status != models.StatusRebasing {
+			if passedFirstPushedCommit {
+				commit.Status = models.StatusPushed
+			} else {
+				commit.Status = models.StatusUnpushed
+			}
 		}
 	}
 
@@ -134,8 +145,23 @@ func (self *CommitLoader) GetCommits(opts GetCommitsOptions) ([]*models.Commit,
 		return commits, nil
 	}
 
-	if ancestor != "" {
-		commits = setCommitMergedStatuses(ancestor, commits)
+	if opts.RefToShowDivergenceFrom != "" {
+		sort.SliceStable(commits, func(i, j int) bool {
+			// In the divergence view we want incoming commits to come first
+			return commits[i].Divergence > commits[j].Divergence
+		})
+
+		_, localSectionStart, found := lo.FindIndexOf(commits, func(commit *models.Commit) bool {
+			return commit.Divergence == models.DivergenceLeft
+		})
+		if !found {
+			localSectionStart = len(commits)
+		}
+
+		setCommitMergedStatuses(remoteAncestor, commits[:localSectionStart])
+		setCommitMergedStatuses(ancestor, commits[localSectionStart:])
+	} else {
+		setCommitMergedStatuses(ancestor, commits)
 	}
 
 	return commits, nil
@@ -176,8 +202,8 @@ func (self *CommitLoader) MergeRebasingCommits(commits []*models.Commit) ([]*mod
 // then puts them into a commit object
 // example input:
 // 8ad01fe32fcc20f07bc6693f87aa4977c327f1e1|10 hours ago|Jesse Duffield| (HEAD -> master, tag: v0.15.2)|refresh commits when adding a tag
-func (self *CommitLoader) extractCommitFromLine(line string) *models.Commit {
-	split := strings.SplitN(line, "\x00", 7)
+func (self *CommitLoader) extractCommitFromLine(line string, showDivergence bool) *models.Commit {
+	split := strings.SplitN(line, "\x00", 8)
 
 	sha := split[0]
 	unixTimestamp := split[1]
@@ -186,6 +212,10 @@ func (self *CommitLoader) extractCommitFromLine(line string) *models.Commit {
 	extraInfo := strings.TrimSpace(split[4])
 	parentHashes := split[5]
 	message := split[6]
+	divergence := models.DivergenceNone
+	if showDivergence {
+		divergence = lo.Ternary(split[7] == "<", models.DivergenceLeft, models.DivergenceRight)
+	}
 
 	tags := []string{}
 
@@ -219,14 +249,12 @@ func (self *CommitLoader) extractCommitFromLine(line string) *models.Commit {
 		AuthorName:    authorName,
 		AuthorEmail:   authorEmail,
 		Parents:       parents,
+		Divergence:    divergence,
 	}
 }
 
 func (self *CommitLoader) getHydratedRebasingCommits(rebaseMode enums.RebaseMode) ([]*models.Commit, error) {
-	commits, err := self.getRebasingCommits(rebaseMode)
-	if err != nil {
-		return nil, err
-	}
+	commits := self.getRebasingCommits(rebaseMode)
 
 	if len(commits) == 0 {
 		return nil, nil
@@ -247,8 +275,8 @@ func (self *CommitLoader) getHydratedRebasingCommits(rebaseMode enums.RebaseMode
 	).DontLog()
 
 	fullCommits := map[string]*models.Commit{}
-	err = cmdObj.RunAndProcessLines(func(line string) (bool, error) {
-		commit := self.extractCommitFromLine(line)
+	err := cmdObj.RunAndProcessLines(func(line string) (bool, error) {
+		commit := self.extractCommitFromLine(line, false)
 		fullCommits[commit.Sha] = commit
 		return false, nil
 	})
@@ -283,73 +311,20 @@ func (self *CommitLoader) getHydratedRebasingCommits(rebaseMode enums.RebaseMode
 }
 
 // getRebasingCommits obtains the commits that we're in the process of rebasing
-func (self *CommitLoader) getRebasingCommits(rebaseMode enums.RebaseMode) ([]*models.Commit, error) {
-	switch rebaseMode {
-	case enums.REBASE_MODE_MERGING:
-		return self.getNormalRebasingCommits()
-	case enums.REBASE_MODE_INTERACTIVE:
-		return self.getInteractiveRebasingCommits()
-	default:
-		return nil, nil
-	}
-}
-
-func (self *CommitLoader) getNormalRebasingCommits() ([]*models.Commit, error) {
-	rewrittenCount := 0
-	bytesContent, err := self.readFile(filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-apply/rewritten"))
-	if err == nil {
-		content := string(bytesContent)
-		rewrittenCount = len(strings.Split(content, "\n"))
-	}
-
-	// we know we're rebasing, so lets get all the files whose names have numbers
-	commits := []*models.Commit{}
-	err = self.walkFiles(filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-apply"), func(path string, f os.FileInfo, err error) error {
-		if rewrittenCount > 0 {
-			rewrittenCount--
-			return nil
-		}
-		if err != nil {
-			return err
-		}
-		re := regexp.MustCompile(`^\d+$`)
-		if !re.MatchString(f.Name()) {
-			return nil
-		}
-		bytesContent, err := self.readFile(path)
-		if err != nil {
-			return err
-		}
-		content := string(bytesContent)
-		commit := self.commitFromPatch(content)
-		commits = append([]*models.Commit{commit}, commits...)
-		return nil
-	})
-	if err != nil {
-		return nil, err
-	}
-
-	return commits, nil
-}
 
 // git-rebase-todo example:
 // pick ac446ae94ee560bdb8d1d057278657b251aaef17 ac446ae
 // pick afb893148791a2fbd8091aeb81deba4930c73031 afb8931
+func (self *CommitLoader) getRebasingCommits(rebaseMode enums.RebaseMode) []*models.Commit {
+	if rebaseMode != enums.REBASE_MODE_INTERACTIVE {
+		return nil
+	}
 
-// git-rebase-todo.backup example:
-// pick 49cbba374296938ea86bbd4bf4fee2f6ba5cccf6 third commit on master
-// pick ac446ae94ee560bdb8d1d057278657b251aaef17 blah  commit on master
-// pick afb893148791a2fbd8091aeb81deba4930c73031 fourth commit on master
-
-// getInteractiveRebasingCommits takes our git-rebase-todo and our git-rebase-todo.backup files
-// and extracts out the sha and names of commits that we still have to go
-// in the rebase:
-func (self *CommitLoader) getInteractiveRebasingCommits() ([]*models.Commit, error) {
 	bytesContent, err := self.readFile(filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/git-rebase-todo"))
 	if err != nil {
 		self.Log.Error(fmt.Sprintf("error occurred reading git-rebase-todo: %s", err.Error()))
 		// we assume an error means the file doesn't exist so we just return
-		return nil, nil
+		return nil
 	}
 
 	commits := []*models.Commit{}
@@ -357,7 +332,7 @@ func (self *CommitLoader) getInteractiveRebasingCommits() ([]*models.Commit, err
 	todos, err := todo.Parse(bytes.NewBuffer(bytesContent), self.config.GetCoreCommentChar())
 	if err != nil {
 		self.Log.Error(fmt.Sprintf("error occurred while parsing git-rebase-todo file: %s", err.Error()))
-		return nil, nil
+		return nil
 	}
 
 	// See if the current commit couldn't be applied because it conflicted; if
@@ -373,7 +348,7 @@ func (self *CommitLoader) getInteractiveRebasingCommits() ([]*models.Commit, err
 
 	for _, t := range todos {
 		if t.Command == todo.UpdateRef {
-			t.Msg = strings.TrimPrefix(t.Ref, "refs/heads/")
+			t.Msg = t.Ref
 		} else if t.Commit == "" {
 			// Command does not have a commit associated, skip
 			continue
@@ -386,7 +361,7 @@ func (self *CommitLoader) getInteractiveRebasingCommits() ([]*models.Commit, err
 		})
 	}
 
-	return commits, nil
+	return commits
 }
 
 func (self *CommitLoader) getConflictedCommit(todos []todo.Todo) string {
@@ -476,23 +451,11 @@ func (self *CommitLoader) getConflictedCommitImpl(todos []todo.Todo, doneTodos [
 	return lastTodo.Commit
 }
 
-// assuming the file starts like this:
-// From e93d4193e6dd45ca9cf3a5a273d7ba6cd8b8fb20 Mon Sep 17 00:00:00 2001
-// From: Lazygit Tester <test@example.com>
-// Date: Wed, 5 Dec 2018 21:03:23 +1100
-// Subject: second commit on master
-func (self *CommitLoader) commitFromPatch(content string) *models.Commit {
-	lines := strings.Split(content, "\n")
-	sha := strings.Split(lines[0], " ")[1]
-	name := strings.TrimPrefix(lines[3], "Subject: ")
-	return &models.Commit{
-		Sha:    sha,
-		Name:   name,
-		Status: models.StatusRebasing,
+func setCommitMergedStatuses(ancestor string, commits []*models.Commit) {
+	if ancestor == "" {
+		return
 	}
-}
 
-func setCommitMergedStatuses(ancestor string, commits []*models.Commit) []*models.Commit {
 	passedAncestor := false
 	for i, commit := range commits {
 		// some commits aren't really commits and don't have sha's, such as the update-ref todo
@@ -506,7 +469,6 @@ func setCommitMergedStatuses(ancestor string, commits []*models.Commit) []*model
 			commits[i].Status = models.StatusMerged
 		}
 	}
-	return commits
 }
 
 func (self *CommitLoader) getMergeBase(refName string) string {
@@ -617,18 +579,25 @@ func (self *CommitLoader) getFirstPushedCommit(refName string) (string, error) {
 
 // getLog gets the git log.
 func (self *CommitLoader) getLogCmd(opts GetCommitsOptions) oscommands.ICmdObj {
-	config := self.UserConfig.Git.Log
+	gitLogOrder := self.AppState.GitLogOrder
+
+	refSpec := opts.RefName
+	if opts.RefToShowDivergenceFrom != "" {
+		refSpec += "..." + opts.RefToShowDivergenceFrom
+	}
 
 	cmdArgs := NewGitCmd("log").
-		Arg(opts.RefName).
-		ArgIf(config.Order != "default", "--"+config.Order).
+		Arg(refSpec).
+		ArgIf(gitLogOrder != "default", "--"+gitLogOrder).
 		ArgIf(opts.All, "--all").
 		Arg("--oneline").
 		Arg(prettyFormat).
 		Arg("--abbrev=40").
+		ArgIf(opts.FilterAuthor != "", "--author="+opts.FilterAuthor).
 		ArgIf(opts.Limit, "-300").
 		ArgIf(opts.FilterPath != "", "--follow").
 		Arg("--no-show-signature").
+		ArgIf(opts.RefToShowDivergenceFrom != "", "--left-right").
 		Arg("--").
 		ArgIf(opts.FilterPath != "", opts.FilterPath).
 		ToArgv()
@@ -636,4 +605,4 @@ func (self *CommitLoader) getLogCmd(opts GetCommitsOptions) oscommands.ICmdObj {
 	return self.cmd.New(cmdArgs).DontLog()
 }
 
-const prettyFormat = `--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%s`
+const prettyFormat = `--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%s%x00%m`
