@@ -79,6 +79,12 @@ func (self *RebaseCommands) SetCommitAuthor(commits []*models.Commit, index int,
 	})
 }
 
+func (self *RebaseCommands) AddCommitCoAuthor(commits []*models.Commit, index int, value string) error {
+	return self.GenericAmend(commits, index, func() error {
+		return self.commit.AddCoAuthor(commits[index].Sha, value)
+	})
+}
+
 func (self *RebaseCommands) GenericAmend(commits []*models.Commit, index int, f func() error) error {
 	if models.IsHeadCommit(commits, index) {
 		// we've selected the top commit so no rebase is required
@@ -99,58 +105,49 @@ func (self *RebaseCommands) GenericAmend(commits []*models.Commit, index int, f
 	return self.ContinueRebase()
 }
 
-func (self *RebaseCommands) MoveCommitDown(commits []*models.Commit, index int) error {
-	baseShaOrRoot := getBaseShaOrRoot(commits, index+2)
+func (self *RebaseCommands) MoveCommitsDown(commits []*models.Commit, startIdx int, endIdx int) error {
+	baseShaOrRoot := getBaseShaOrRoot(commits, endIdx+2)
 
-	sha := commits[index].Sha
-
-	msg := utils.ResolvePlaceholderString(
-		self.Tr.Log.MoveCommitDown,
-		map[string]string{
-			"shortSha": utils.ShortSha(sha),
-		},
-	)
-	self.os.LogCommand(msg, false)
+	shas := lo.Map(commits[startIdx:endIdx+1], func(commit *models.Commit, _ int) string {
+		return commit.Sha
+	})
 
 	return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
 		baseShaOrRoot:  baseShaOrRoot,
-		instruction:    daemon.NewMoveTodoDownInstruction(sha),
+		instruction:    daemon.NewMoveTodosDownInstruction(shas),
 		overrideEditor: true,
 	}).Run()
 }
 
-func (self *RebaseCommands) MoveCommitUp(commits []*models.Commit, index int) error {
-	baseShaOrRoot := getBaseShaOrRoot(commits, index+1)
+func (self *RebaseCommands) MoveCommitsUp(commits []*models.Commit, startIdx int, endIdx int) error {
+	baseShaOrRoot := getBaseShaOrRoot(commits, endIdx+1)
 
-	sha := commits[index].Sha
-
-	msg := utils.ResolvePlaceholderString(
-		self.Tr.Log.MoveCommitUp,
-		map[string]string{
-			"shortSha": utils.ShortSha(sha),
-		},
-	)
-	self.os.LogCommand(msg, false)
+	shas := lo.Map(commits[startIdx:endIdx+1], func(commit *models.Commit, _ int) string {
+		return commit.Sha
+	})
 
 	return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
 		baseShaOrRoot:  baseShaOrRoot,
-		instruction:    daemon.NewMoveTodoUpInstruction(sha),
+		instruction:    daemon.NewMoveTodosUpInstruction(shas),
 		overrideEditor: true,
 	}).Run()
 }
 
-func (self *RebaseCommands) InteractiveRebase(commits []*models.Commit, index int, action todo.TodoCommand) error {
-	baseIndex := index + 1
+func (self *RebaseCommands) InteractiveRebase(commits []*models.Commit, startIdx int, endIdx int, action todo.TodoCommand) error {
+	baseIndex := endIdx + 1
 	if action == todo.Squash || action == todo.Fixup {
 		baseIndex++
 	}
 
 	baseShaOrRoot := getBaseShaOrRoot(commits, baseIndex)
 
-	changes := []daemon.ChangeTodoAction{{
-		Sha:       commits[index].Sha,
-		NewAction: action,
-	}}
+	changes := lo.Map(commits[startIdx:endIdx+1], func(commit *models.Commit, _ int) daemon.ChangeTodoAction {
+		return daemon.ChangeTodoAction{
+			Sha:       commit.Sha,
+			NewAction: action,
+		}
+	})
+
 	self.os.LogCommand(logTodoChanges(changes), false)
 
 	return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
@@ -194,7 +191,7 @@ func logTodoChanges(changes []daemon.ChangeTodoAction) string {
 	changeTodoStr := strings.Join(lo.Map(changes, func(c daemon.ChangeTodoAction, _ int) string {
 		return fmt.Sprintf("%s:%s", c.Sha, c.NewAction)
 	}), "\n")
-	return fmt.Sprintf("Changing TODO actions: %s", changeTodoStr)
+	return fmt.Sprintf("Changing TODO actions:\n%s", changeTodoStr)
 }
 
 type PrepareInteractiveRebaseCommandOpts struct {
@@ -215,9 +212,9 @@ func (self *RebaseCommands) PrepareInteractiveRebaseCommand(opts PrepareInteract
 		Arg("--interactive").
 		Arg("--autostash").
 		Arg("--keep-empty").
-		ArgIf(opts.keepCommitsThatBecomeEmpty && !self.version.IsOlderThan(2, 26, 0), "--empty=keep").
+		ArgIf(opts.keepCommitsThatBecomeEmpty && self.version.IsAtLeast(2, 26, 0), "--empty=keep").
 		Arg("--no-autosquash").
-		ArgIf(!self.version.IsOlderThan(2, 22, 0), "--rebase-merges").
+		ArgIf(self.version.IsAtLeast(2, 22, 0), "--rebase-merges").
 		ArgIf(opts.onto != "", "--onto", opts.onto).
 		Arg(opts.baseShaOrRoot).
 		ToArgv()
@@ -275,22 +272,59 @@ func (self *RebaseCommands) AmendTo(commits []*models.Commit, commitIndex int) e
 	}).Run()
 }
 
-// EditRebaseTodo sets the action for a given rebase commit in the git-rebase-todo file
-func (self *RebaseCommands) EditRebaseTodo(commit *models.Commit, action todo.TodoCommand) error {
+func todoFromCommit(commit *models.Commit) utils.Todo {
+	if commit.Action == todo.UpdateRef {
+		return utils.Todo{Ref: commit.Name, Action: commit.Action}
+	} else {
+		return utils.Todo{Sha: commit.Sha, Action: commit.Action}
+	}
+}
+
+// Sets the action for the given commits in the git-rebase-todo file
+func (self *RebaseCommands) EditRebaseTodo(commits []*models.Commit, action todo.TodoCommand) error {
+	commitsWithAction := lo.Map(commits, func(commit *models.Commit, _ int) utils.TodoChange {
+		return utils.TodoChange{
+			Sha:       commit.Sha,
+			OldAction: commit.Action,
+			NewAction: action,
+		}
+	})
+
 	return utils.EditRebaseTodo(
-		filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/git-rebase-todo"), commit.Sha, commit.Action, action, self.config.GetCoreCommentChar())
+		filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/git-rebase-todo"),
+		commitsWithAction,
+		self.config.GetCoreCommentChar(),
+	)
 }
 
-// MoveTodoDown moves a rebase todo item down by one position
-func (self *RebaseCommands) MoveTodoDown(commit *models.Commit) error {
-	fileName := filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/git-rebase-todo")
-	return utils.MoveTodoDown(fileName, commit.Sha, commit.Action, self.config.GetCoreCommentChar())
+func (self *RebaseCommands) DeleteUpdateRefTodos(commits []*models.Commit) error {
+	todosToDelete := lo.Map(commits, func(commit *models.Commit, _ int) utils.Todo {
+		return todoFromCommit(commit)
+	})
+
+	return utils.DeleteTodos(
+		filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/git-rebase-todo"),
+		todosToDelete,
+		self.config.GetCoreCommentChar(),
+	)
 }
 
-// MoveTodoDown moves a rebase todo item down by one position
-func (self *RebaseCommands) MoveTodoUp(commit *models.Commit) error {
+func (self *RebaseCommands) MoveTodosDown(commits []*models.Commit) error {
 	fileName := filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/git-rebase-todo")
-	return utils.MoveTodoUp(fileName, commit.Sha, commit.Action, self.config.GetCoreCommentChar())
+	todosToMove := lo.Map(commits, func(commit *models.Commit, _ int) utils.Todo {
+		return todoFromCommit(commit)
+	})
+
+	return utils.MoveTodosDown(fileName, todosToMove, self.config.GetCoreCommentChar())
+}
+
+func (self *RebaseCommands) MoveTodosUp(commits []*models.Commit) error {
+	fileName := filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/git-rebase-todo")
+	todosToMove := lo.Map(commits, func(commit *models.Commit, _ int) utils.Todo {
+		return todoFromCommit(commit)
+	})
+
+	return utils.MoveTodosUp(fileName, todosToMove, self.config.GetCoreCommentChar())
 }
 
 // SquashAllAboveFixupCommits squashes all fixup! commits above the given one
@@ -403,23 +437,25 @@ func (self *RebaseCommands) runSkipEditorCommand(cmdObj oscommands.ICmdObj) erro
 }
 
 // DiscardOldFileChanges discards changes to a file from an old commit
-func (self *RebaseCommands) DiscardOldFileChanges(commits []*models.Commit, commitIndex int, fileName string) error {
+func (self *RebaseCommands) DiscardOldFileChanges(commits []*models.Commit, commitIndex int, filePaths []string) error {
 	if err := self.BeginInteractiveRebaseForCommit(commits, commitIndex, false); err != nil {
 		return err
 	}
 
-	// check if file exists in previous commit (this command returns an error if the file doesn't exist)
-	cmdArgs := NewGitCmd("cat-file").Arg("-e", "HEAD^:"+fileName).ToArgv()
+	for _, filePath := range filePaths {
+		// check if file exists in previous commit (this command returns an error if the file doesn't exist)
+		cmdArgs := NewGitCmd("cat-file").Arg("-e", "HEAD^:"+filePath).ToArgv()
 
-	if err := self.cmd.New(cmdArgs).Run(); err != nil {
-		if err := self.os.Remove(fileName); err != nil {
+		if err := self.cmd.New(cmdArgs).Run(); err != nil {
+			if err := self.os.Remove(filePath); err != nil {
+				return err
+			}
+			if err := self.workingTree.StageFile(filePath); err != nil {
+				return err
+			}
+		} else if err := self.workingTree.CheckoutFile("HEAD^", filePath); err != nil {
 			return err
 		}
-		if err := self.workingTree.StageFile(fileName); err != nil {
-			return err
-		}
-	} else if err := self.workingTree.CheckoutFile("HEAD^", fileName); err != nil {
-		return err
 	}
 
 	// amend the commit
@@ -451,6 +487,20 @@ func (self *RebaseCommands) CherryPickCommits(commits []*models.Commit) error {
 	}).Run()
 }
 
+// CherryPickCommitsDuringRebase simply prepends the given commits to the existing git-rebase-todo file
+func (self *RebaseCommands) CherryPickCommitsDuringRebase(commits []*models.Commit) error {
+	todoLines := lo.Map(commits, func(commit *models.Commit, _ int) daemon.TodoLine {
+		return daemon.TodoLine{
+			Action: "pick",
+			Commit: commit,
+		}
+	})
+
+	todo := daemon.TodoLinesToString(todoLines)
+	filePath := filepath.Join(self.repoPaths.worktreeGitDirPath, "rebase-merge/git-rebase-todo")
+	return utils.PrependStrToTodoFile(filePath, []byte(todo))
+}
+
 // we can't start an interactive rebase from the first commit without passing the
 // '--root' arg
 func getBaseShaOrRoot(commits []*models.Commit, index int) string {
