@@ -3,15 +3,10 @@ package git_commands
 import (
 	"fmt"
 	"os"
-	"path/filepath"
-	"strings"
-	"time"
 
 	"github.com/go-errors/errors"
-	"github.com/jesseduffield/generics/slices"
 	"github.com/jesseduffield/lazygit/pkg/commands/models"
 	"github.com/jesseduffield/lazygit/pkg/commands/oscommands"
-	"github.com/jesseduffield/lazygit/pkg/utils"
 )
 
 type WorkingTreeCommands struct {
@@ -33,11 +28,7 @@ func NewWorkingTreeCommands(
 }
 
 func (self *WorkingTreeCommands) OpenMergeToolCmdObj() oscommands.ICmdObj {
-	return self.cmd.New("git mergetool")
-}
-
-func (self *WorkingTreeCommands) OpenMergeTool() error {
-	return self.OpenMergeToolCmdObj().Run()
+	return self.cmd.New(NewGitCmd("mergetool").ToArgv())
 }
 
 // StageFile stages a file
@@ -46,38 +37,40 @@ func (self *WorkingTreeCommands) StageFile(path string) error {
 }
 
 func (self *WorkingTreeCommands) StageFiles(paths []string) error {
-	quotedPaths := slices.Map(paths, func(path string) string {
-		return self.cmd.Quote(path)
-	})
-	return self.cmd.New(fmt.Sprintf("git add -- %s", strings.Join(quotedPaths, " "))).Run()
+	cmdArgs := NewGitCmd("add").Arg("--").Arg(paths...).ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // StageAll stages all files
 func (self *WorkingTreeCommands) StageAll() error {
-	return self.cmd.New("git add -A").Run()
+	cmdArgs := NewGitCmd("add").Arg("-A").ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // UnstageAll unstages all files
 func (self *WorkingTreeCommands) UnstageAll() error {
-	return self.cmd.New("git reset").Run()
+	return self.cmd.New(NewGitCmd("reset").ToArgv()).Run()
 }
 
 // UnStageFile unstages a file
 // we accept an array of filenames for the cases where a file has been renamed i.e.
 // we accept the current name and the previous name
-func (self *WorkingTreeCommands) UnStageFile(fileNames []string, reset bool) error {
-	command := "git rm --cached --force -- %s"
-	if reset {
-		command = "git reset HEAD -- %s"
+func (self *WorkingTreeCommands) UnStageFile(paths []string, tracked bool) error {
+	if tracked {
+		return self.UnstageTrackedFiles(paths)
+	} else {
+		return self.UnstageUntrackedFiles(paths)
 	}
+}
 
-	for _, name := range fileNames {
-		err := self.cmd.New(fmt.Sprintf(command, self.cmd.Quote(name))).Run()
-		if err != nil {
-			return err
-		}
-	}
-	return nil
+func (self *WorkingTreeCommands) UnstageTrackedFiles(paths []string) error {
+	return self.cmd.New(NewGitCmd("reset").Arg("HEAD", "--").Arg(paths...).ToArgv()).Run()
+}
+
+func (self *WorkingTreeCommands) UnstageUntrackedFiles(paths []string) error {
+	return self.cmd.New(NewGitCmd("rm").Arg("--cached", "--force", "--").Arg(paths...).ToArgv()).Run()
 }
 
 func (self *WorkingTreeCommands) BeforeAndAfterFileForRename(file *models.File) (*models.File, *models.File, error) {
@@ -134,25 +127,32 @@ func (self *WorkingTreeCommands) DiscardAllFileChanges(file *models.File) error
 		return nil
 	}
 
-	quotedFileName := self.cmd.Quote(file.Name)
-
 	if file.ShortStatus == "AA" {
-		if err := self.cmd.New("git checkout --ours --  " + quotedFileName).Run(); err != nil {
+		if err := self.cmd.New(
+			NewGitCmd("checkout").Arg("--ours", "--", file.Name).ToArgv(),
+		).Run(); err != nil {
 			return err
 		}
-		if err := self.cmd.New("git add -- " + quotedFileName).Run(); err != nil {
+
+		if err := self.cmd.New(
+			NewGitCmd("add").Arg("--", file.Name).ToArgv(),
+		).Run(); err != nil {
 			return err
 		}
 		return nil
 	}
 
 	if file.ShortStatus == "DU" {
-		return self.cmd.New("git rm -- " + quotedFileName).Run()
+		return self.cmd.New(
+			NewGitCmd("rm").Arg("--", file.Name).ToArgv(),
+		).Run()
 	}
 
 	// if the file isn't tracked, we assume you want to delete it
 	if file.HasStagedChanges || file.HasMergeConflicts {
-		if err := self.cmd.New("git reset -- " + quotedFileName).Run(); err != nil {
+		if err := self.cmd.New(
+			NewGitCmd("reset").Arg("--", file.Name).ToArgv(),
+		).Run(); err != nil {
 			return err
 		}
 	}
@@ -164,6 +164,7 @@ func (self *WorkingTreeCommands) DiscardAllFileChanges(file *models.File) error
 	if file.Added {
 		return self.os.RemoveFile(file.Name)
 	}
+
 	return self.DiscardUnstagedFileChanges(file)
 }
 
@@ -171,6 +172,8 @@ type IFileNode interface {
 	ForEachFile(cb func(*models.File) error) error
 	GetFilePathsMatching(test func(*models.File) bool) []string
 	GetPath() string
+	// Returns file if the node is not a directory, otherwise returns nil
+	GetFile() *models.File
 }
 
 func (self *WorkingTreeCommands) DiscardAllDirChanges(node IFileNode) error {
@@ -179,13 +182,24 @@ func (self *WorkingTreeCommands) DiscardAllDirChanges(node IFileNode) error {
 }
 
 func (self *WorkingTreeCommands) DiscardUnstagedDirChanges(node IFileNode) error {
-	if err := self.RemoveUntrackedDirFiles(node); err != nil {
-		return err
-	}
+	file := node.GetFile()
+	if file == nil {
+		if err := self.RemoveUntrackedDirFiles(node); err != nil {
+			return err
+		}
 
-	quotedPath := self.cmd.Quote(node.GetPath())
-	if err := self.cmd.New("git checkout -- " + quotedPath).Run(); err != nil {
-		return err
+		cmdArgs := NewGitCmd("checkout").Arg("--", node.GetPath()).ToArgv()
+		if err := self.cmd.New(cmdArgs).Run(); err != nil {
+			return err
+		}
+	} else {
+		if file.Added && !file.HasStagedChanges {
+			return self.os.RemoveFile(file.Name)
+		}
+
+		if err := self.DiscardUnstagedFileChanges(file); err != nil {
+			return err
+		}
 	}
 
 	return nil
@@ -206,10 +220,9 @@ func (self *WorkingTreeCommands) RemoveUntrackedDirFiles(node IFileNode) error {
 	return nil
 }
 
-// DiscardUnstagedFileChanges directly
 func (self *WorkingTreeCommands) DiscardUnstagedFileChanges(file *models.File) error {
-	quotedFileName := self.cmd.Quote(file.Name)
-	return self.cmd.New("git checkout -- " + quotedFileName).Run()
+	cmdArgs := NewGitCmd("checkout").Arg("--", file.Name).ToArgv()
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // Ignore adds a file to the gitignore for the repo
@@ -223,127 +236,114 @@ func (self *WorkingTreeCommands) Exclude(filename string) error {
 }
 
 // WorktreeFileDiff returns the diff of a file
-func (self *WorkingTreeCommands) WorktreeFileDiff(file *models.File, plain bool, cached bool, ignoreWhitespace bool) string {
+func (self *WorkingTreeCommands) WorktreeFileDiff(file *models.File, plain bool, cached bool) string {
 	// for now we assume an error means the file was deleted
-	s, _ := self.WorktreeFileDiffCmdObj(file, plain, cached, ignoreWhitespace).RunWithOutput()
+	s, _ := self.WorktreeFileDiffCmdObj(file, plain, cached).RunWithOutput()
 	return s
 }
 
-func (self *WorkingTreeCommands) WorktreeFileDiffCmdObj(node models.IFile, plain bool, cached bool, ignoreWhitespace bool) oscommands.ICmdObj {
-	cachedArg := ""
-	trackedArg := "--"
+func (self *WorkingTreeCommands) WorktreeFileDiffCmdObj(node models.IFile, plain bool, cached bool) oscommands.ICmdObj {
 	colorArg := self.UserConfig.Git.Paging.ColorArg
-	quotedPath := self.cmd.Quote(node.GetPath())
-	quotedPrevPath := ""
-	ignoreWhitespaceArg := ""
-	contextSize := self.UserConfig.Git.DiffContextSize
-	if cached {
-		cachedArg = " --cached"
-	}
-	if !node.GetIsTracked() && !node.GetHasStagedChanges() && !cached && node.GetIsFile() {
-		trackedArg = "--no-index -- /dev/null"
-	}
 	if plain {
 		colorArg = "never"
 	}
-	if ignoreWhitespace {
-		ignoreWhitespaceArg = " --ignore-all-space"
-	}
-	if prevPath := node.GetPreviousPath(); prevPath != "" {
-		quotedPrevPath = " " + self.cmd.Quote(prevPath)
-	}
 
-	cmdStr := fmt.Sprintf("git diff --submodule --no-ext-diff --unified=%d --color=%s%s%s %s %s%s", contextSize, colorArg, ignoreWhitespaceArg, cachedArg, trackedArg, quotedPath, quotedPrevPath)
+	contextSize := self.AppState.DiffContextSize
+	prevPath := node.GetPreviousPath()
+	noIndex := !node.GetIsTracked() && !node.GetHasStagedChanges() && !cached && node.GetIsFile()
+	extDiffCmd := self.UserConfig.Git.Paging.ExternalDiffCommand
+	useExtDiff := extDiffCmd != "" && !plain
 
-	return self.cmd.New(cmdStr).DontLog()
-}
+	cmdArgs := NewGitCmd("diff").
+		ConfigIf(useExtDiff, "diff.external="+extDiffCmd).
+		ArgIfElse(useExtDiff, "--ext-diff", "--no-ext-diff").
+		Arg("--submodule").
+		Arg(fmt.Sprintf("--unified=%d", contextSize)).
+		Arg(fmt.Sprintf("--color=%s", colorArg)).
+		ArgIf(!plain && self.AppState.IgnoreWhitespaceInDiffView, "--ignore-all-space").
+		ArgIf(cached, "--cached").
+		ArgIf(noIndex, "--no-index").
+		Arg("--").
+		ArgIf(noIndex, "/dev/null").
+		Arg(node.GetPath()).
+		ArgIf(prevPath != "", prevPath).
+		Dir(self.repoPaths.worktreePath).
+		ToArgv()
 
-func (self *WorkingTreeCommands) ApplyPatch(patch string, flags ...string) error {
-	filepath, err := self.SaveTemporaryPatch(patch)
-	if err != nil {
-		return err
-	}
-
-	return self.ApplyPatchFile(filepath, flags...)
-}
-
-func (self *WorkingTreeCommands) ApplyPatchFile(filepath string, flags ...string) error {
-	flagStr := ""
-	for _, flag := range flags {
-		flagStr += " --" + flag
-	}
-
-	return self.cmd.New(fmt.Sprintf("git apply%s %s", flagStr, self.cmd.Quote(filepath))).Run()
-}
-
-func (self *WorkingTreeCommands) SaveTemporaryPatch(patch string) (string, error) {
-	filepath := filepath.Join(self.os.GetTempDir(), utils.GetCurrentRepoName(), time.Now().Format("Jan _2 15.04.05.000000000")+".patch")
-	self.Log.Infof("saving temporary patch to %s", filepath)
-	if err := self.os.CreateFileWithContent(filepath, patch); err != nil {
-		return "", err
-	}
-	return filepath, nil
+	return self.cmd.New(cmdArgs).DontLog()
 }
 
 // ShowFileDiff get the diff of specified from and to. Typically this will be used for a single commit so it'll be 123abc^..123abc
 // but when we're in diff mode it could be any 'from' to any 'to'. The reverse flag is also here thanks to diff mode.
-func (self *WorkingTreeCommands) ShowFileDiff(from string, to string, reverse bool, fileName string, plain bool,
-	ignoreWhitespace bool,
-) (string, error) {
-	return self.ShowFileDiffCmdObj(from, to, reverse, fileName, plain, ignoreWhitespace).RunWithOutput()
+func (self *WorkingTreeCommands) ShowFileDiff(from string, to string, reverse bool, fileName string, plain bool) (string, error) {
+	return self.ShowFileDiffCmdObj(from, to, reverse, fileName, plain).RunWithOutput()
 }
 
-func (self *WorkingTreeCommands) ShowFileDiffCmdObj(from string, to string, reverse bool, fileName string, plain bool,
-	ignoreWhitespace bool,
-) oscommands.ICmdObj {
+func (self *WorkingTreeCommands) ShowFileDiffCmdObj(from string, to string, reverse bool, fileName string, plain bool) oscommands.ICmdObj {
+	contextSize := self.AppState.DiffContextSize
+
 	colorArg := self.UserConfig.Git.Paging.ColorArg
-	contextSize := self.UserConfig.Git.DiffContextSize
 	if plain {
 		colorArg = "never"
 	}
 
-	reverseFlag := ""
-	if reverse {
-		reverseFlag = " -R"
-	}
+	extDiffCmd := self.UserConfig.Git.Paging.ExternalDiffCommand
+	useExtDiff := extDiffCmd != "" && !plain
 
-	ignoreWhitespaceFlag := ""
-	if ignoreWhitespace {
-		ignoreWhitespaceFlag = " --ignore-all-space"
-	}
+	cmdArgs := NewGitCmd("diff").
+		Config("diff.noprefix=false").
+		ConfigIf(useExtDiff, "diff.external="+extDiffCmd).
+		ArgIfElse(useExtDiff, "--ext-diff", "--no-ext-diff").
+		Arg("--submodule").
+		Arg(fmt.Sprintf("--unified=%d", contextSize)).
+		Arg("--no-renames").
+		Arg(fmt.Sprintf("--color=%s", colorArg)).
+		Arg(from).
+		Arg(to).
+		ArgIf(reverse, "-R").
+		ArgIf(!plain && self.AppState.IgnoreWhitespaceInDiffView, "--ignore-all-space").
+		Arg("--").
+		Arg(fileName).
+		Dir(self.repoPaths.worktreePath).
+		ToArgv()
 
-	return self.cmd.
-		New(
-			fmt.Sprintf(
-				"git diff --submodule --no-ext-diff --unified=%d --no-renames --color=%s%s%s%s%s -- %s",
-				contextSize, colorArg, pad(from), pad(to), reverseFlag, ignoreWhitespaceFlag, self.cmd.Quote(fileName)),
-		).
-		DontLog()
+	return self.cmd.New(cmdArgs).DontLog()
 }
 
 // CheckoutFile checks out the file for the given commit
 func (self *WorkingTreeCommands) CheckoutFile(commitSha, fileName string) error {
-	return self.cmd.New(fmt.Sprintf("git checkout %s -- %s", commitSha, self.cmd.Quote(fileName))).Run()
+	cmdArgs := NewGitCmd("checkout").Arg(commitSha, "--", fileName).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
-// DiscardAnyUnstagedFileChanges discards any unstages file changes via `git checkout -- .`
+// DiscardAnyUnstagedFileChanges discards any unstaged file changes via `git checkout -- .`
 func (self *WorkingTreeCommands) DiscardAnyUnstagedFileChanges() error {
-	return self.cmd.New("git checkout -- .").Run()
+	cmdArgs := NewGitCmd("checkout").Arg("--", ".").
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // RemoveTrackedFiles will delete the given file(s) even if they are currently tracked
 func (self *WorkingTreeCommands) RemoveTrackedFiles(name string) error {
-	return self.cmd.New("git rm -r --cached -- " + self.cmd.Quote(name)).Run()
+	cmdArgs := NewGitCmd("rm").Arg("-r", "--cached", "--", name).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // RemoveUntrackedFiles runs `git clean -fd`
 func (self *WorkingTreeCommands) RemoveUntrackedFiles() error {
-	return self.cmd.New("git clean -fd").Run()
+	cmdArgs := NewGitCmd("clean").Arg("-fd").ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // ResetAndClean removes all unstaged changes and removes all untracked files
 func (self *WorkingTreeCommands) ResetAndClean() error {
-	submoduleConfigs, err := self.submodule.GetConfigs()
+	submoduleConfigs, err := self.submodule.GetConfigs(nil)
 	if err != nil {
 		return err
 	}
@@ -363,23 +363,23 @@ func (self *WorkingTreeCommands) ResetAndClean() error {
 
 // ResetHardHead runs `git reset --hard`
 func (self *WorkingTreeCommands) ResetHard(ref string) error {
-	return self.cmd.New("git reset --hard " + self.cmd.Quote(ref)).Run()
+	cmdArgs := NewGitCmd("reset").Arg("--hard", ref).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // ResetSoft runs `git reset --soft HEAD`
 func (self *WorkingTreeCommands) ResetSoft(ref string) error {
-	return self.cmd.New("git reset --soft " + self.cmd.Quote(ref)).Run()
+	cmdArgs := NewGitCmd("reset").Arg("--soft", ref).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 func (self *WorkingTreeCommands) ResetMixed(ref string) error {
-	return self.cmd.New("git reset --mixed " + self.cmd.Quote(ref)).Run()
-}
+	cmdArgs := NewGitCmd("reset").Arg("--mixed", ref).
+		ToArgv()
 
-// so that we don't have unnecessary space in our commands we use this helper function to prepend spaces to args so that in the format string we can go '%s%s%s' and if any args are missing we won't have gaps.
-func pad(str string) string {
-	if str == "" {
-		return ""
-	}
-
-	return " " + str
+	return self.cmd.New(cmdArgs).Run()
 }
