@@ -28,13 +28,13 @@ func GetEditAtLineAndWaitTemplate(osConfig *OSConfig, guessDefaultEditor func()
 	return template
 }
 
-func GetOpenDirInEditorTemplate(osConfig *OSConfig, guessDefaultEditor func() string) string {
+func GetOpenDirInEditorTemplate(osConfig *OSConfig, guessDefaultEditor func() string) (string, bool) {
 	preset := getPreset(osConfig, guessDefaultEditor)
 	template := osConfig.OpenDirInEditor
 	if template == "" {
 		template = preset.openDirInEditorTemplate
 	}
-	return template
+	return template, getEditInTerminal(osConfig, preset)
 }
 
 type editPreset struct {
@@ -42,62 +42,80 @@ type editPreset struct {
 	editAtLineTemplate        string
 	editAtLineAndWaitTemplate string
 	openDirInEditorTemplate   string
-	editInTerminal            bool
+	suspend                   bool
 }
 
 // IF YOU ADD A PRESET TO THIS FUNCTION YOU MUST UPDATE THE `Supported presets` SECTION OF docs/Config.md
 func getPreset(osConfig *OSConfig, guessDefaultEditor func() string) *editPreset {
 	presets := map[string]*editPreset{
-		"vi":      standardTerminalEditorPreset("vi"),
-		"vim":     standardTerminalEditorPreset("vim"),
-		"nvim":    standardTerminalEditorPreset("nvim"),
+		"vi":   standardTerminalEditorPreset("vi"),
+		"vim":  standardTerminalEditorPreset("vim"),
+		"nvim": standardTerminalEditorPreset("nvim"),
+		"nvim-remote": {
+			editTemplate:       `nvim --server "$NVIM" --remote-tab {{filename}}`,
+			editAtLineTemplate: `nvim --server "$NVIM" --remote-tab {{filename}}; [ -z "$NVIM" ] || nvim --server "$NVIM" --remote-send ":{{line}}<CR>"`,
+			// No remote-wait support yet. See https://github.com/neovim/neovim/pull/17856
+			editAtLineAndWaitTemplate: `nvim +{{line}} {{filename}}`,
+			openDirInEditorTemplate:   `nvim --server "$NVIM" --remote-tab {{dir}}`,
+			suspend:                   false,
+		},
+		"lvim":    standardTerminalEditorPreset("lvim"),
 		"emacs":   standardTerminalEditorPreset("emacs"),
+		"micro":   standardTerminalEditorPreset("micro"),
 		"nano":    standardTerminalEditorPreset("nano"),
 		"kakoune": standardTerminalEditorPreset("kak"),
 		"helix": {
+			editTemplate:              "helix -- {{filename}}",
+			editAtLineTemplate:        "helix -- {{filename}}:{{line}}",
+			editAtLineAndWaitTemplate: "helix -- {{filename}}:{{line}}",
+			openDirInEditorTemplate:   "helix -- {{dir}}",
+			suspend:                   true,
+		},
+		"helix (hx)": {
 			editTemplate:              "hx -- {{filename}}",
 			editAtLineTemplate:        "hx -- {{filename}}:{{line}}",
 			editAtLineAndWaitTemplate: "hx -- {{filename}}:{{line}}",
 			openDirInEditorTemplate:   "hx -- {{dir}}",
-			editInTerminal:            true,
+			suspend:                   true,
 		},
 		"vscode": {
 			editTemplate:              "code --reuse-window -- {{filename}}",
 			editAtLineTemplate:        "code --reuse-window --goto -- {{filename}}:{{line}}",
 			editAtLineAndWaitTemplate: "code --reuse-window --goto --wait -- {{filename}}:{{line}}",
 			openDirInEditorTemplate:   "code -- {{dir}}",
-			editInTerminal:            false,
+			suspend:                   false,
 		},
 		"sublime": {
 			editTemplate:              "subl -- {{filename}}",
 			editAtLineTemplate:        "subl -- {{filename}}:{{line}}",
 			editAtLineAndWaitTemplate: "subl --wait -- {{filename}}:{{line}}",
 			openDirInEditorTemplate:   "subl -- {{dir}}",
-			editInTerminal:            false,
+			suspend:                   false,
 		},
 		"bbedit": {
 			editTemplate:              "bbedit -- {{filename}}",
 			editAtLineTemplate:        "bbedit +{{line}} -- {{filename}}",
 			editAtLineAndWaitTemplate: "bbedit +{{line}} --wait -- {{filename}}",
 			openDirInEditorTemplate:   "bbedit -- {{dir}}",
-			editInTerminal:            false,
+			suspend:                   false,
 		},
 		"xcode": {
 			editTemplate:              "xed -- {{filename}}",
 			editAtLineTemplate:        "xed --line {{line}} -- {{filename}}",
 			editAtLineAndWaitTemplate: "xed --line {{line}} --wait -- {{filename}}",
 			openDirInEditorTemplate:   "xed -- {{dir}}",
-			editInTerminal:            false,
+			suspend:                   false,
 		},
 	}
 
 	// Some of our presets have a different name than the editor they are using.
 	editorToPreset := map[string]string{
-		"kak":  "kakoune",
-		"hx":   "helix",
-		"code": "vscode",
-		"subl": "sublime",
-		"xed":  "xcode",
+		"kak":   "kakoune",
+		"helix": "helix",
+		"hx":    "helix (hx)",
+		"code":  "vscode",
+		"subl":  "sublime",
+		"xed":   "xcode",
 	}
 
 	presetName := osConfig.EditPreset
@@ -123,13 +141,13 @@ func standardTerminalEditorPreset(editor string) *editPreset {
 		editAtLineTemplate:        editor + " +{{line}} -- {{filename}}",
 		editAtLineAndWaitTemplate: editor + " +{{line}} -- {{filename}}",
 		openDirInEditorTemplate:   editor + " -- {{dir}}",
-		editInTerminal:            true,
+		suspend:                   true,
 	}
 }
 
 func getEditInTerminal(osConfig *OSConfig, preset *editPreset) bool {
-	if osConfig.EditInTerminal != nil {
-		return *osConfig.EditInTerminal
+	if osConfig.SuspendOnEdit != nil {
+		return *osConfig.SuspendOnEdit
 	}
-	return preset.editInTerminal
+	return preset.suspend
 }
