@@ -20,10 +20,11 @@ type BaseContext struct {
 	onFocusFn           onFocusFn
 	onFocusLostFn       onFocusLostFn
 
-	focusable           bool
-	transient           bool
-	hasControlledBounds bool
-	highlightOnFocus    bool
+	focusable                  bool
+	transient                  bool
+	hasControlledBounds        bool
+	needsRerenderOnWidthChange bool
+	highlightOnFocus           bool
 
 	*ParentContextMgr
 }
@@ -36,14 +37,15 @@ type (
 var _ types.IBaseContext = &BaseContext{}
 
 type NewBaseContextOpts struct {
-	Kind                  types.ContextKind
-	Key                   types.ContextKey
-	View                  *gocui.View
-	WindowName            string
-	Focusable             bool
-	Transient             bool
-	HasUncontrolledBounds bool // negating for the sake of making false the default
-	HighlightOnFocus      bool
+	Kind                       types.ContextKind
+	Key                        types.ContextKey
+	View                       *gocui.View
+	WindowName                 string
+	Focusable                  bool
+	Transient                  bool
+	HasUncontrolledBounds      bool // negating for the sake of making false the default
+	HighlightOnFocus           bool
+	NeedsRerenderOnWidthChange bool
 
 	OnGetOptionsMap func() map[string]string
 }
@@ -54,17 +56,18 @@ func NewBaseContext(opts NewBaseContextOpts) *BaseContext {
 	hasControlledBounds := !opts.HasUncontrolledBounds
 
 	return &BaseContext{
-		kind:                opts.Kind,
-		key:                 opts.Key,
-		view:                opts.View,
-		windowName:          opts.WindowName,
-		onGetOptionsMap:     opts.OnGetOptionsMap,
-		focusable:           opts.Focusable,
-		transient:           opts.Transient,
-		hasControlledBounds: hasControlledBounds,
-		highlightOnFocus:    opts.HighlightOnFocus,
-		ParentContextMgr:    &ParentContextMgr{},
-		viewTrait:           viewTrait,
+		kind:                       opts.Kind,
+		key:                        opts.Key,
+		view:                       opts.View,
+		windowName:                 opts.WindowName,
+		onGetOptionsMap:            opts.OnGetOptionsMap,
+		focusable:                  opts.Focusable,
+		transient:                  opts.Transient,
+		hasControlledBounds:        hasControlledBounds,
+		highlightOnFocus:           opts.HighlightOnFocus,
+		needsRerenderOnWidthChange: opts.NeedsRerenderOnWidthChange,
+		ParentContextMgr:           &ParentContextMgr{},
+		viewTrait:                  viewTrait,
 	}
 }
 
@@ -190,6 +193,10 @@ func (self *BaseContext) HasControlledBounds() bool {
 	return self.hasControlledBounds
 }
 
+func (self *BaseContext) NeedsRerenderOnWidthChange() bool {
+	return self.needsRerenderOnWidthChange
+}
+
 func (self *BaseContext) Title() string {
 	return ""
 }
