@@ -34,24 +34,17 @@ func NewMenuContext(
 				Focusable:             true,
 				HasUncontrolledBounds: true,
 			})),
-			getDisplayStrings:   viewModel.GetDisplayStrings,
-			list:                viewModel,
-			c:                   c,
-			getColumnAlignments: func() []utils.Alignment { return viewModel.columnAlignment },
+			ListRenderer: ListRenderer{
+				list:                viewModel,
+				getDisplayStrings:   viewModel.GetDisplayStrings,
+				getColumnAlignments: func() []utils.Alignment { return viewModel.columnAlignment },
+				getNonModelItems:    viewModel.GetNonModelItems,
+			},
+			c: c,
 		},
 	}
 }
 
-// TODO: remove this thing.
-func (self *MenuContext) GetSelectedItemId() string {
-	item := self.GetSelected()
-	if item == nil {
-		return ""
-	}
-
-	return item.Label
-}
-
 type MenuViewModel struct {
 	c               *ContextCommon
 	menuItems       []*types.MenuItem
@@ -79,7 +72,7 @@ func (self *MenuViewModel) SetMenuItems(items []*types.MenuItem, columnAlignment
 }
 
 // TODO: move into presentation package
-func (self *MenuViewModel) GetDisplayStrings(_startIdx int, _length int) [][]string {
+func (self *MenuViewModel) GetDisplayStrings(_ int, _ int) [][]string {
 	menuItems := self.FilteredListViewModel.GetItems()
 	showKeys := lo.SomeBy(menuItems, func(item *types.MenuItem) bool {
 		return item.Key != nil
@@ -87,30 +80,54 @@ func (self *MenuViewModel) GetDisplayStrings(_startIdx int, _length int) [][]str
 
 	return lo.Map(menuItems, func(item *types.MenuItem, _ int) []string {
 		displayStrings := item.LabelColumns
+		if item.DisabledReason != nil {
+			displayStrings[0] = style.FgDefault.SetStrikethrough().Sprint(displayStrings[0])
+		}
 
 		if !showKeys {
 			return displayStrings
 		}
 
-		// These keys are used for general navigation so we'll strike them out to
-		// avoid confusion
-		reservedKeys := []string{
-			self.c.UserConfig.Keybinding.Universal.Confirm,
-			self.c.UserConfig.Keybinding.Universal.Select,
-			self.c.UserConfig.Keybinding.Universal.Return,
-			self.c.UserConfig.Keybinding.Universal.StartSearch,
-		}
 		keyLabel := keybindings.LabelFromKey(item.Key)
-		keyStyle := style.FgCyan
-		if lo.Contains(reservedKeys, keyLabel) {
-			keyStyle = style.FgDefault.SetStrikethrough()
-		}
-
-		displayStrings = utils.Prepend(displayStrings, keyStyle.Sprint(keyLabel))
+		displayStrings = utils.Prepend(displayStrings, style.FgCyan.Sprint(keyLabel))
 		return displayStrings
 	})
 }
 
+func (self *MenuViewModel) GetNonModelItems() []*NonModelItem {
+	// Don't display section headers when we are filtering, and the filter mode
+	// is fuzzy. The reason is that filtering changes the order of the items
+	// (they are sorted by best match), so all the sections would be messed up.
+	if self.FilteredListViewModel.IsFiltering() && self.c.UserConfig.Gui.UseFuzzySearch() {
+		return []*NonModelItem{}
+	}
+
+	result := []*NonModelItem{}
+	menuItems := self.FilteredListViewModel.GetItems()
+	var prevSection *types.MenuSection = nil
+	for i, menuItem := range menuItems {
+		menuItem := menuItem
+		if menuItem.Section != nil && menuItem.Section != prevSection {
+			if prevSection != nil {
+				result = append(result, &NonModelItem{
+					Index:   i,
+					Column:  1,
+					Content: "",
+				})
+			}
+
+			result = append(result, &NonModelItem{
+				Index:   i,
+				Column:  1,
+				Content: style.FgGreen.SetBold().Sprintf("--- %s ---", menuItem.Section.Title),
+			})
+			prevSection = menuItem.Section
+		}
+	}
+
+	return result
+}
+
 func (self *MenuContext) GetKeybindings(opts types.KeybindingsOpts) []*types.Binding {
 	basicBindings := self.ListContextTrait.GetKeybindings(opts)
 	menuItemsWithKeys := lo.Filter(self.menuItems, func(item *types.MenuItem, _ int) bool {
@@ -132,6 +149,15 @@ func (self *MenuContext) GetKeybindings(opts types.KeybindingsOpts) []*types.Bin
 }
 
 func (self *MenuContext) OnMenuPress(selectedItem *types.MenuItem) error {
+	if selectedItem != nil && selectedItem.DisabledReason != nil {
+		if selectedItem.DisabledReason.ShowErrorInPanel {
+			return self.c.ErrorMsg(selectedItem.DisabledReason.Text)
+		}
+
+		self.c.ErrorToast(self.c.Tr.DisabledMenuItemPrefix + selectedItem.DisabledReason.Text)
+		return nil
+	}
+
 	if err := self.c.PopContext(); err != nil {
 		return err
 	}
@@ -146,3 +172,8 @@ func (self *MenuContext) OnMenuPress(selectedItem *types.MenuItem) error {
 
 	return nil
 }
+
+// There is currently no need to use range-select in a menu so we're disabling it.
+func (self *MenuContext) RangeSelectEnabled() bool {
+	return false
+}
