@@ -84,28 +84,19 @@ func (self *WorkingTreeHelper) OpenMergeTool() error {
 }
 
 func (self *WorkingTreeHelper) HandleCommitPressWithMessage(initialMessage string) error {
-	if err := self.prepareFilesForCommit(); err != nil {
-		return self.c.Error(err)
-	}
-
-	if len(self.c.Model().Files) == 0 {
-		return self.c.ErrorMsg(self.c.Tr.NoFilesStagedTitle)
-	}
-
-	if !self.AnyStagedFiles() {
-		return self.PromptToStageAllAndRetry(self.HandleCommitPress)
-	}
-
-	return self.commitsHelper.OpenCommitMessagePanel(
-		&OpenCommitMessagePanelOpts{
-			CommitIndex:      context.NoCommitIndex,
-			InitialMessage:   initialMessage,
-			SummaryTitle:     self.c.Tr.CommitSummaryTitle,
-			DescriptionTitle: self.c.Tr.CommitDescriptionTitle,
-			PreserveMessage:  true,
-			OnConfirm:        self.handleCommit,
-		},
-	)
+	return self.WithEnsureCommitableFiles(func() error {
+		return self.commitsHelper.OpenCommitMessagePanel(
+			&OpenCommitMessagePanelOpts{
+				CommitIndex:      context.NoCommitIndex,
+				InitialMessage:   initialMessage,
+				SummaryTitle:     self.c.Tr.CommitSummaryTitle,
+				DescriptionTitle: self.c.Tr.CommitDescriptionTitle,
+				PreserveMessage:  true,
+				OnConfirm:        self.handleCommit,
+				OnSwitchToEditor: self.switchFromCommitMessagePanelToEditor,
+			},
+		)
+	})
 }
 
 func (self *WorkingTreeHelper) handleCommit(summary string, description string) error {
@@ -117,23 +108,32 @@ func (self *WorkingTreeHelper) handleCommit(summary string, description string)
 	})
 }
 
-// HandleCommitEditorPress - handle when the user wants to commit changes via
-// their editor rather than via the popup panel
-func (self *WorkingTreeHelper) HandleCommitEditorPress() error {
-	if len(self.c.Model().Files) == 0 {
-		return self.c.ErrorMsg(self.c.Tr.NoFilesStagedTitle)
-	}
-
-	if !self.AnyStagedFiles() {
-		return self.PromptToStageAllAndRetry(self.HandleCommitEditorPress)
-	}
+func (self *WorkingTreeHelper) switchFromCommitMessagePanelToEditor(filepath string) error {
+	// We won't be able to tell whether the commit was successful, because
+	// RunSubprocessAndRefresh doesn't return the error (it opens an error alert
+	// itself and returns nil on error). But even if we could, we wouldn't have
+	// access to the last message that the user typed, and it might be very
+	// different from what was last in the commit panel. So the best we can do
+	// here is to always clear the remembered commit message.
+	self.commitsHelper.OnCommitSuccess()
 
 	self.c.LogAction(self.c.Tr.Actions.Commit)
 	return self.c.RunSubprocessAndRefresh(
-		self.c.Git().Commit.CommitEditorCmdObj(),
+		self.c.Git().Commit.CommitInEditorWithMessageFileCmdObj(filepath),
 	)
 }
 
+// HandleCommitEditorPress - handle when the user wants to commit changes via
+// their editor rather than via the popup panel
+func (self *WorkingTreeHelper) HandleCommitEditorPress() error {
+	return self.WithEnsureCommitableFiles(func() error {
+		self.c.LogAction(self.c.Tr.Actions.Commit)
+		return self.c.RunSubprocessAndRefresh(
+			self.c.Git().Commit.CommitEditorCmdObj(),
+		)
+	})
+}
+
 func (self *WorkingTreeHelper) HandleWIPCommitPress() error {
 	skipHookPrefix := self.c.UserConfig.Git.SkipHookPrefix
 	if skipHookPrefix == "" {
@@ -163,7 +163,23 @@ func (self *WorkingTreeHelper) HandleCommitPress() error {
 	return self.HandleCommitPressWithMessage(message)
 }
 
-func (self *WorkingTreeHelper) PromptToStageAllAndRetry(retry func() error) error {
+func (self *WorkingTreeHelper) WithEnsureCommitableFiles(handler func() error) error {
+	if err := self.prepareFilesForCommit(); err != nil {
+		return self.c.Error(err)
+	}
+
+	if len(self.c.Model().Files) == 0 {
+		return self.c.ErrorMsg(self.c.Tr.NoFilesStagedTitle)
+	}
+
+	if !self.AnyStagedFiles() {
+		return self.promptToStageAllAndRetry(handler)
+	}
+
+	return handler()
+}
+
+func (self *WorkingTreeHelper) promptToStageAllAndRetry(retry func() error) error {
 	return self.c.Confirm(types.ConfirmOpts{
 		Title:  self.c.Tr.NoFilesStagedTitle,
 		Prompt: self.c.Tr.NoFilesStagedPrompt,
