@@ -9,46 +9,65 @@ import (
 
 type StashController struct {
 	baseController
+	*ListControllerTrait[*models.StashEntry]
 	c *ControllerCommon
 }
 
 var _ types.IController = &StashController{}
 
 func NewStashController(
-	common *ControllerCommon,
+	c *ControllerCommon,
 ) *StashController {
 	return &StashController{
 		baseController: baseController{},
-		c:              common,
+		ListControllerTrait: NewListControllerTrait[*models.StashEntry](
+			c,
+			c.Contexts().Stash,
+			c.Contexts().Stash.GetSelected,
+			c.Contexts().Stash.GetSelectedItems,
+		),
+		c: c,
 	}
 }
 
 func (self *StashController) GetKeybindings(opts types.KeybindingsOpts) []*types.Binding {
 	bindings := []*types.Binding{
 		{
-			Key:         opts.GetKey(opts.Config.Universal.Select),
-			Handler:     self.checkSelected(self.handleStashApply),
-			Description: self.c.Tr.Apply,
+			Key:               opts.GetKey(opts.Config.Universal.Select),
+			Handler:           self.withItem(self.handleStashApply),
+			GetDisabledReason: self.require(self.singleItemSelected()),
+			Description:       self.c.Tr.Apply,
+			Tooltip:           self.c.Tr.StashApplyTooltip,
+			DisplayOnScreen:   true,
 		},
 		{
-			Key:         opts.GetKey(opts.Config.Stash.PopStash),
-			Handler:     self.checkSelected(self.handleStashPop),
-			Description: self.c.Tr.Pop,
+			Key:               opts.GetKey(opts.Config.Stash.PopStash),
+			Handler:           self.withItem(self.handleStashPop),
+			GetDisabledReason: self.require(self.singleItemSelected()),
+			Description:       self.c.Tr.Pop,
+			Tooltip:           self.c.Tr.StashPopTooltip,
+			DisplayOnScreen:   true,
 		},
 		{
-			Key:         opts.GetKey(opts.Config.Universal.Remove),
-			Handler:     self.checkSelected(self.handleStashDrop),
-			Description: self.c.Tr.Drop,
+			Key:               opts.GetKey(opts.Config.Universal.Remove),
+			Handler:           self.withItem(self.handleStashDrop),
+			GetDisabledReason: self.require(self.singleItemSelected()),
+			Description:       self.c.Tr.Drop,
+			Tooltip:           self.c.Tr.StashDropTooltip,
+			DisplayOnScreen:   true,
 		},
 		{
-			Key:         opts.GetKey(opts.Config.Universal.New),
-			Handler:     self.checkSelected(self.handleNewBranchOffStashEntry),
-			Description: self.c.Tr.NewBranch,
+			Key:               opts.GetKey(opts.Config.Universal.New),
+			Handler:           self.withItem(self.handleNewBranchOffStashEntry),
+			GetDisabledReason: self.require(self.singleItemSelected()),
+			Description:       self.c.Tr.NewBranch,
+			Tooltip:           self.c.Tr.NewBranchFromStashTooltip,
 		},
 		{
-			Key:         opts.GetKey(opts.Config.Stash.RenameStash),
-			Handler:     self.checkSelected(self.handleRenameStashEntry),
-			Description: self.c.Tr.RenameStash,
+			Key:               opts.GetKey(opts.Config.Stash.RenameStash),
+			Handler:           self.withItem(self.handleRenameStashEntry),
+			GetDisabledReason: self.require(self.singleItemSelected()),
+			Description:       self.c.Tr.RenameStash,
 		},
 	}
 
@@ -64,10 +83,7 @@ func (self *StashController) GetOnRenderToMain() func() error {
 				task = types.NewRenderStringTask(self.c.Tr.NoStashEntries)
 			} else {
 				task = types.NewRunPtyTask(
-					self.c.Git().Stash.ShowStashEntryCmdObj(
-						stashEntry.Index,
-						self.c.GetAppState().IgnoreWhitespaceInDiffView,
-					).GetCmd(),
+					self.c.Git().Stash.ShowStashEntryCmdObj(stashEntry.Index).GetCmd(),
 				)
 			}
 
@@ -83,21 +99,6 @@ func (self *StashController) GetOnRenderToMain() func() error {
 	}
 }
 
-func (self *StashController) checkSelected(callback func(*models.StashEntry) error) func() error {
-	return func() error {
-		item := self.context().GetSelected()
-		if item == nil {
-			return nil
-		}
-
-		return callback(item)
-	}
-}
-
-func (self *StashController) Context() types.Context {
-	return self.context()
-}
-
 func (self *StashController) context() *context.StashContext {
 	return self.c.Contexts().Stash
 }
@@ -192,7 +193,7 @@ func (self *StashController) handleRenameStashEntry(stashEntry *models.StashEntr
 			if err != nil {
 				return err
 			}
-			self.context().SetSelectedLineIdx(0) // Select the renamed stash
+			self.context().SetSelection(0) // Select the renamed stash
 			self.context().FocusLine()
 			return nil
 		},
