@@ -24,9 +24,7 @@ func (self *ToggleWhitespaceAction) Call() error {
 	}
 
 	self.c.GetAppState().IgnoreWhitespaceInDiffView = !self.c.GetAppState().IgnoreWhitespaceInDiffView
-	if err := self.c.SaveAppState(); err != nil {
-		self.c.Log.Errorf("error when saving app state: %v", err)
-	}
+	self.c.SaveAppStateAndLogError()
 
 	return self.c.CurrentSideContext().HandleFocus(types.OnFocusOpts{})
 }
