@@ -5,6 +5,7 @@ import (
 	"github.com/jesseduffield/lazygit/pkg/commands"
 	"github.com/jesseduffield/lazygit/pkg/commands/oscommands"
 	"github.com/jesseduffield/lazygit/pkg/config"
+	"github.com/jesseduffield/lazygit/pkg/gui/controllers/helpers"
 	"github.com/jesseduffield/lazygit/pkg/gui/types"
 )
 
@@ -92,6 +93,12 @@ func (self *guiCommon) SaveAppState() error {
 	return self.gui.Config.SaveAppState()
 }
 
+func (self *guiCommon) SaveAppStateAndLogError() {
+	if err := self.gui.Config.SaveAppState(); err != nil {
+		self.gui.Log.Errorf("error when saving app state: %v", err)
+	}
+}
+
 func (self *guiCommon) GetConfig() config.AppConfigurer {
 	return self.gui.Config
 }
@@ -165,6 +172,10 @@ func (self *guiCommon) KeybindingsOpts() types.KeybindingsOpts {
 	return self.gui.keybindingOpts()
 }
 
+func (self *guiCommon) CallKeybindingHandler(binding *types.Binding) error {
+	return self.gui.callKeybindingHandler(binding)
+}
+
 func (self *guiCommon) IsAnyModeActive() bool {
 	return self.gui.helpers.Mode.IsAnyModeActive()
 }
@@ -182,6 +193,15 @@ func (self *guiCommon) AfterLayout(f func() error) {
 	}
 }
 
+func (self *guiCommon) RunningIntegrationTest() bool {
+	return self.gui.integrationTest != nil
+}
+
 func (self *guiCommon) InDemo() bool {
 	return self.gui.integrationTest != nil && self.gui.integrationTest.IsDemo()
 }
+
+func (self *guiCommon) WithInlineStatus(item types.HasUrn, operation types.ItemOperation, contextKey types.ContextKey, f func(gocui.Task) error) error {
+	self.gui.helpers.InlineStatus.WithInlineStatus(helpers.InlineStatusOpts{Item: item, Operation: operation, ContextKey: contextKey}, f)
+	return nil
+}
