@@ -3,11 +3,9 @@ package popup
 import (
 	"context"
 	"strings"
-	"time"
 
 	"github.com/jesseduffield/gocui"
 	"github.com/jesseduffield/lazygit/pkg/common"
-	gctx "github.com/jesseduffield/lazygit/pkg/gui/context"
 	"github.com/jesseduffield/lazygit/pkg/gui/style"
 	"github.com/jesseduffield/lazygit/pkg/gui/types"
 	"github.com/sasha-s/go-deadlock"
@@ -17,16 +15,16 @@ type PopupHandler struct {
 	*common.Common
 	index int
 	deadlock.Mutex
-	createPopupPanelFn  func(context.Context, types.CreatePopupPanelOpts) error
-	onErrorFn           func() error
-	popContextFn        func() error
-	currentContextFn    func() types.Context
-	createMenuFn        func(types.CreateMenuOptions) error
-	withWaitingStatusFn func(message string, f func(gocui.Task) error)
-	toastFn             func(message string)
-	getPromptInputFn    func() string
-	onWorker            func(func(gocui.Task))
-	inDemo              func() bool
+	createPopupPanelFn      func(context.Context, types.CreatePopupPanelOpts) error
+	onErrorFn               func() error
+	popContextFn            func() error
+	currentContextFn        func() types.Context
+	createMenuFn            func(types.CreateMenuOptions) error
+	withWaitingStatusFn     func(message string, f func(gocui.Task) error)
+	withWaitingStatusSyncFn func(message string, f func() error)
+	toastFn                 func(message string, kind types.ToastKind)
+	getPromptInputFn        func() string
+	inDemo                  func() bool
 }
 
 var _ types.IPopupHandler = &PopupHandler{}
@@ -39,24 +37,24 @@ func NewPopupHandler(
 	currentContextFn func() types.Context,
 	createMenuFn func(types.CreateMenuOptions) error,
 	withWaitingStatusFn func(message string, f func(gocui.Task) error),
-	toastFn func(message string),
+	withWaitingStatusSyncFn func(message string, f func() error),
+	toastFn func(message string, kind types.ToastKind),
 	getPromptInputFn func() string,
-	onWorker func(func(gocui.Task)),
 	inDemo func() bool,
 ) *PopupHandler {
 	return &PopupHandler{
-		Common:              common,
-		index:               0,
-		createPopupPanelFn:  createPopupPanelFn,
-		onErrorFn:           onErrorFn,
-		popContextFn:        popContextFn,
-		currentContextFn:    currentContextFn,
-		createMenuFn:        createMenuFn,
-		withWaitingStatusFn: withWaitingStatusFn,
-		toastFn:             toastFn,
-		getPromptInputFn:    getPromptInputFn,
-		onWorker:            onWorker,
-		inDemo:              inDemo,
+		Common:                  common,
+		index:                   0,
+		createPopupPanelFn:      createPopupPanelFn,
+		onErrorFn:               onErrorFn,
+		popContextFn:            popContextFn,
+		currentContextFn:        currentContextFn,
+		createMenuFn:            createMenuFn,
+		withWaitingStatusFn:     withWaitingStatusFn,
+		withWaitingStatusSyncFn: withWaitingStatusSyncFn,
+		toastFn:                 toastFn,
+		getPromptInputFn:        getPromptInputFn,
+		inDemo:                  inDemo,
 	}
 }
 
@@ -65,7 +63,15 @@ func (self *PopupHandler) Menu(opts types.CreateMenuOptions) error {
 }
 
 func (self *PopupHandler) Toast(message string) {
-	self.toastFn(message)
+	self.toastFn(message, types.ToastKindStatus)
+}
+
+func (self *PopupHandler) ErrorToast(message string) {
+	self.toastFn(message, types.ToastKindError)
+}
+
+func (self *PopupHandler) SetToastFunc(f func(string, types.ToastKind)) {
+	self.toastFn = f
 }
 
 func (self *PopupHandler) WithWaitingStatus(message string, f func(gocui.Task) error) error {
@@ -73,6 +79,11 @@ func (self *PopupHandler) WithWaitingStatus(message string, f func(gocui.Task) e
 	return nil
 }
 
+func (self *PopupHandler) WithWaitingStatusSync(message string, f func() error) error {
+	self.withWaitingStatusSyncFn(message, f)
+	return nil
+}
+
 func (self *PopupHandler) Error(err error) error {
 	if err == gocui.ErrQuit {
 		return err
@@ -128,47 +139,6 @@ func (self *PopupHandler) Prompt(opts types.PromptOpts) error {
 	})
 }
 
-func (self *PopupHandler) WithLoaderPanel(message string, f func(gocui.Task) error) error {
-	index := 0
-	self.Lock()
-	self.index++
-	index = self.index
-	self.Unlock()
-
-	ctx, cancel := context.WithCancel(context.Background())
-
-	err := self.createPopupPanelFn(ctx, types.CreatePopupPanelOpts{
-		Prompt:    message,
-		HasLoader: true,
-	})
-	if err != nil {
-		self.Log.Error(err)
-		cancel()
-		return nil
-	}
-
-	self.onWorker(func(task gocui.Task) {
-		// emulating a delay due to network latency
-		if self.inDemo() {
-			time.Sleep(500 * time.Millisecond)
-		}
-
-		if err := f(task); err != nil {
-			self.Log.Error(err)
-		}
-
-		cancel()
-
-		self.Lock()
-		if index == self.index && self.currentContextFn().GetKey() == gctx.CONFIRMATION_CONTEXT_KEY {
-			_ = self.popContextFn()
-		}
-		self.Unlock()
-	})
-
-	return nil
-}
-
 // returns the content that has currently been typed into the prompt. Useful for
 // asynchronously updating the suggestions list under the prompt.
 func (self *PopupHandler) GetPromptInput() string {
