@@ -3,15 +3,18 @@ package presentation
 import (
 	"fmt"
 	"strings"
+	"time"
 
 	"github.com/jesseduffield/lazygit/pkg/commands/git_commands"
 	"github.com/jesseduffield/lazygit/pkg/commands/models"
 	"github.com/jesseduffield/lazygit/pkg/config"
 	"github.com/jesseduffield/lazygit/pkg/gui/presentation/icons"
 	"github.com/jesseduffield/lazygit/pkg/gui/style"
+	"github.com/jesseduffield/lazygit/pkg/gui/types"
 	"github.com/jesseduffield/lazygit/pkg/i18n"
 	"github.com/jesseduffield/lazygit/pkg/theme"
 	"github.com/jesseduffield/lazygit/pkg/utils"
+	"github.com/mattn/go-runewidth"
 	"github.com/samber/lo"
 )
 
@@ -19,27 +22,52 @@ var branchPrefixColorCache = make(map[string]style.TextStyle)
 
 func GetBranchListDisplayStrings(
 	branches []*models.Branch,
+	getItemOperation func(item types.HasUrn) types.ItemOperation,
 	fullDescription bool,
 	diffName string,
+	viewWidth int,
 	tr *i18n.TranslationSet,
 	userConfig *config.UserConfig,
 	worktrees []*models.Worktree,
 ) [][]string {
 	return lo.Map(branches, func(branch *models.Branch, _ int) []string {
 		diffed := branch.Name == diffName
-		return getBranchDisplayStrings(branch, fullDescription, diffed, tr, userConfig, worktrees)
+		return getBranchDisplayStrings(branch, getItemOperation(branch), fullDescription, diffed, viewWidth, tr, userConfig, worktrees, time.Now())
 	})
 }
 
 // getBranchDisplayStrings returns the display string of branch
 func getBranchDisplayStrings(
 	b *models.Branch,
+	itemOperation types.ItemOperation,
 	fullDescription bool,
 	diffed bool,
+	viewWidth int,
 	tr *i18n.TranslationSet,
 	userConfig *config.UserConfig,
 	worktrees []*models.Worktree,
+	now time.Time,
 ) []string {
+	checkedOutByWorkTree := git_commands.CheckedOutByOtherWorktree(b, worktrees)
+	showCommitHash := fullDescription || userConfig.Gui.ShowBranchCommitHash
+	branchStatus := BranchStatus(b, itemOperation, tr, now)
+	worktreeIcon := lo.Ternary(icons.IsIconEnabled(), icons.LINKED_WORKTREE_ICON, fmt.Sprintf("(%s)", tr.LcWorktree))
+
+	// Recency is always three characters, plus one for the space
+	availableWidth := viewWidth - 4
+	if len(branchStatus) > 0 {
+		availableWidth -= runewidth.StringWidth(branchStatus) + 1
+	}
+	if icons.IsIconEnabled() {
+		availableWidth -= 2 // one for the icon, one for the space
+	}
+	if showCommitHash {
+		availableWidth -= utils.COMMIT_HASH_SHORT_SIZE + 1
+	}
+	if checkedOutByWorkTree {
+		availableWidth -= runewidth.StringWidth(worktreeIcon) + 1
+	}
+
 	displayName := b.Name
 	if b.DisplayName != "" {
 		displayName = b.DisplayName
@@ -50,13 +78,20 @@ func getBranchDisplayStrings(
 		nameTextStyle = theme.DiffTerminalColor
 	}
 
+	// Don't bother shortening branch names that are already 3 characters or less
+	if len(displayName) > utils.Max(availableWidth, 3) {
+		// Never shorten the branch name to less then 3 characters
+		len := utils.Max(availableWidth, 4)
+		displayName = displayName[:len-1] + "…"
+	}
 	coloredName := nameTextStyle.Sprint(displayName)
-	branchStatus := utils.WithPadding(ColoredBranchStatus(b, tr), 2, utils.AlignLeft)
-	if git_commands.CheckedOutByOtherWorktree(b, worktrees) {
-		worktreeIcon := lo.Ternary(icons.IsIconEnabled(), icons.LINKED_WORKTREE_ICON, fmt.Sprintf("(%s)", tr.LcWorktree))
+	if checkedOutByWorkTree {
 		coloredName = fmt.Sprintf("%s %s", coloredName, style.FgDefault.Sprint(worktreeIcon))
 	}
-	coloredName = fmt.Sprintf("%s %s", coloredName, branchStatus)
+	if len(branchStatus) > 0 {
+		coloredStatus := branchStatusColor(b, itemOperation).Sprint(branchStatus)
+		coloredName = fmt.Sprintf("%s %s", coloredName, coloredStatus)
+	}
 
 	recencyColor := style.FgCyan
 	if b.Recency == "  *" {
@@ -70,7 +105,7 @@ func getBranchDisplayStrings(
 		res = append(res, nameTextStyle.Sprint(icons.IconForBranch(b)))
 	}
 
-	if fullDescription || userConfig.Gui.ShowBranchCommitHash {
+	if showCommitHash {
 		res = append(res, utils.ShortSha(b.CommitHash))
 	}
 
@@ -109,9 +144,11 @@ func GetBranchTextStyle(name string) style.TextStyle {
 	}
 }
 
-func ColoredBranchStatus(branch *models.Branch, tr *i18n.TranslationSet) string {
+func branchStatusColor(branch *models.Branch, itemOperation types.ItemOperation) style.TextStyle {
 	colour := style.FgYellow
-	if branch.UpstreamGone {
+	if itemOperation != types.ItemOperationNone {
+		colour = style.FgCyan
+	} else if branch.UpstreamGone {
 		colour = style.FgRed
 	} else if branch.MatchesUpstream() {
 		colour = style.FgGreen
@@ -119,10 +156,19 @@ func ColoredBranchStatus(branch *models.Branch, tr *i18n.TranslationSet) string
 		colour = style.FgMagenta
 	}
 
-	return colour.Sprint(BranchStatus(branch, tr))
+	return colour
 }
 
-func BranchStatus(branch *models.Branch, tr *i18n.TranslationSet) string {
+func ColoredBranchStatus(branch *models.Branch, itemOperation types.ItemOperation, tr *i18n.TranslationSet) string {
+	return branchStatusColor(branch, itemOperation).Sprint(BranchStatus(branch, itemOperation, tr, time.Now()))
+}
+
+func BranchStatus(branch *models.Branch, itemOperation types.ItemOperation, tr *i18n.TranslationSet, now time.Time) string {
+	itemOperationStr := ItemOperationToString(itemOperation, tr)
+	if itemOperationStr != "" {
+		return itemOperationStr + " " + utils.Loader(now)
+	}
+
 	if !branch.IsTrackingRemote() {
 		return ""
 	}
