@@ -45,8 +45,9 @@ func (gui *Gui) onResize() error {
 func (gui *Gui) newPtyTask(view *gocui.View, cmd *exec.Cmd, prefix string) error {
 	width, _ := gui.Views.Main.Size()
 	pager := gui.git.Config.GetPager(width)
+	externalDiffCommand := gui.Config.GetUserConfig().Git.Paging.ExternalDiffCommand
 
-	if pager == "" {
+	if pager == "" && externalDiffCommand == "" {
 		// if we're not using a custom pager we don't need to use a pty
 		return gui.newCmdTask(view, cmd, prefix)
 	}
@@ -55,9 +56,6 @@ func (gui *Gui) newPtyTask(view *gocui.View, cmd *exec.Cmd, prefix string) error
 
 	cmd.Env = append(cmd.Env, "GIT_PAGER="+pager)
 
-	_, height := view.Size()
-	_, oy := view.Origin()
-
 	manager := gui.getManager(view)
 
 	var ptmx *os.File
@@ -82,7 +80,8 @@ func (gui *Gui) newPtyTask(view *gocui.View, cmd *exec.Cmd, prefix string) error
 		gui.Mutexes.PtyMutex.Unlock()
 	}
 
-	if err := manager.NewTask(manager.NewCmdTask(start, prefix, height+oy+10, onClose), cmdStr); err != nil {
+	linesToRead := gui.linesToReadFromCmdTask(view)
+	if err := manager.NewTask(manager.NewCmdTask(start, prefix, linesToRead, onClose), cmdStr); err != nil {
 		return err
 	}
 
