@@ -39,7 +39,7 @@ func japaneseTranslationSet() TranslationSet {
 		CredentialsPassword:     "パスワード",
 		CredentialsPassphrase:   "SSH鍵のパスフレーズを入力",
 		PassUnameWrong:          "パスワード, パスフレーズまたはユーザ名が間違っています。",
-		CommitChanges:           "変更をコミット",
+		Commit:                  "変更をコミット",
 		AmendLastCommit:         "最新のコミットにamend",
 		AmendLastCommitTitle:    "最新のコミットにamend",
 		SureToAmend:             "最新のコミットに変更をamendします。よろしいですか? コミットメッセージはコミットパネルから変更できます。",
@@ -48,7 +48,7 @@ func japaneseTranslationSet() TranslationSet {
 		StatusTitle:             "ステータス",
 		Menu:                    "メニュー",
 		Execute:                 "実行",
-		ToggleStaged:            "ステージ/アンステージ",
+		Stage:                   "ステージ/アンステージ",
 		ToggleStagedAll:         "すべての変更をステージ/アンステージ",
 		ToggleTreeView:          "ファイルツリーの表示を切り替え",
 		OpenMergeTool:           "Git mergetoolを開く",
@@ -63,9 +63,6 @@ func japaneseTranslationSet() TranslationSet {
 		FilterUnstagedFiles:     "ステージされていないファイルのみを表示",
 		ResetFilter:             "フィルタをリセット",
 		// NoChangedFiles:                      "No changed files",
-		PullWait:                "Pull中...",
-		PushWait:                "Push中...",
-		FetchWait:               "Fetch中...",
 		SoftReset:               "Softリセット",
 		AlreadyCheckedOutBranch: "ブランチはすでにチェックアウトされています。",
 		// SureForceCheckout:                   "Are you sure you want force checkout? You will lose all local changes",
@@ -73,9 +70,6 @@ func japaneseTranslationSet() TranslationSet {
 		BranchName:               "ブランチ名",
 		NewBranchNameBranchOff:   "新規ブランチ名 ('{{.branchName}}' に作成)",
 		CantDeleteCheckOutBranch: "チェックアウト中のブランチは削除できません!",
-		DeleteBranch:             "ブランチを削除",
-		DeleteBranchMessage:      "ブランチ '{{.selectedBranchName}}' を削除します。よろしいですか?",
-		ForceDeleteBranchMessage: "'{{.selectedBranchName}}' はマージされていません。本当に削除しますか?",
 		// LcRebaseBranch:                      "Rebase checked-out branch onto this branch",
 		CantRebaseOntoSelf:        "ブランチを自分自身にリベースすることはできません。",
 		CantMergeBranchIntoItself: "ブランチを自分自身にマージすることはできません。",
@@ -88,7 +82,6 @@ func japaneseTranslationSet() TranslationSet {
 		Confirm:                 "確認",
 		Close:                   "閉じる",
 		Quit:                    "終了",
-		// LcSquashDown:                        "Squash down",
 		// LcFixupCommit:                       "Fixup commit",
 		// NoCommitsThisBranch:                 "No commits for this branch",
 		// CannotSquashOrFixupFirstCommit:      "There's no commit below to squash into",
@@ -98,13 +91,13 @@ func japaneseTranslationSet() TranslationSet {
 		// Squash:                              "Squash",
 		// LcPickCommit:                        "Pick commit (when mid-rebase)",
 		RevertCommit:       "コミットをrevert",
-		RewordCommit:       "コミットメッセージを変更",
-		DeleteCommit:       "コミットを削除",
+		Reword:             "コミットメッセージを変更",
+		DropCommit:         "コミットを削除",
 		MoveDownCommit:     "コミットを1つ下に移動",
 		MoveUpCommit:       "コミットを1つ上に移動",
-		EditCommit:         "コミットを編集",
-		AmendToCommit:      "ステージされた変更でamendコミット",
-		RenameCommitEditor: "エディタでコミットメッセージを編集",
+		EditCommitTooltip:  "コミットを編集",
+		AmendCommitTooltip: "ステージされた変更でamendコミット",
+		RewordCommitEditor: "エディタでコミットメッセージを編集",
 		Error:              "エラー",
 		// LcPickHunk:                          "Pick hunk",
 		// LcPickAllHunks:                      "Pick all hunks",
@@ -149,29 +142,29 @@ func japaneseTranslationSet() TranslationSet {
 		MergeToolPrompt:                  "`git mergetool`を開きます。よろしいですか?",
 		IntroPopupMessage:                japaneseIntroPopupMessage,
 		// GitconfigParseErr:                   `Gogit failed to parse your gitconfig file due to the presence of unquoted '\' characters. Removing these should fix the issue.`,
-		EditFile:               `ファイルを編集`,
-		OpenFile:               `ファイルを開く`,
-		IgnoreFile:             `.gitignoreに追加`,
-		RefreshFiles:           `ファイルをリフレッシュ`,
-		MergeIntoCurrentBranch: `現在のブランチにマージ`,
-		ConfirmQuit:            `終了します。よろしいですか?`,
-		SwitchRepo:             `最近使用したリポジトリに切り替え`,
-		AllBranchesLogGraph:    `すべてのブランチログを表示`,
-		UnsupportedGitService:  `サポートされていないGitサービスです。`,
-		CreatePullRequest:      `Pull Requestを作成`,
-		CopyPullRequestURL:     `Pull RequestのURLをクリップボードにコピー`,
-		NoBranchOnRemote:       `ブランチがリモートに存在しません。リモートにpushしてください。`,
-		Fetch:                  `Fetch`,
+		EditFile:              `ファイルを編集`,
+		OpenFile:              `ファイルを開く`,
+		IgnoreFile:            `.gitignoreに追加`,
+		RefreshFiles:          `ファイルをリフレッシュ`,
+		Merge:                 `現在のブランチにマージ`,
+		ConfirmQuit:           `終了します。よろしいですか?`,
+		SwitchRepo:            `最近使用したリポジトリに切り替え`,
+		AllBranchesLogGraph:   `すべてのブランチログを表示`,
+		UnsupportedGitService: `サポートされていないGitサービスです。`,
+		CreatePullRequest:     `Pull Requestを作成`,
+		CopyPullRequestURL:    `Pull RequestのURLをクリップボードにコピー`,
+		NoBranchOnRemote:      `ブランチがリモートに存在しません。リモートにpushしてください。`,
+		Fetch:                 `Fetch`,
 		// NoAutomaticGitFetchTitle:            `No automatic git fetch`,
 		// NoAutomaticGitFetchBody:             `Lazygit can't use "git fetch" in a private repo; use 'f' in the files panel to run "git fetch" manually`,
 		// FileEnter:                           `stage individual hunks/lines for file, or collapse/expand for directory`,
 		// FileStagingRequirements:             `Can only stage individual lines for tracked files`,
-		StageSelection:          `選択行をステージ/アンステージ`,
+		StageSelectionTooltip:   `選択行をステージ/アンステージ`,
 		DiscardSelection:        `変更を削除 (git reset)`,
-		ToggleDragSelect:        `範囲選択を切り替え`,
+		ToggleRangeSelect:       `範囲選択を切り替え`,
 		ToggleSelectHunk:        `Hunk選択を切り替え`,
 		ToggleSelectionForPatch: `行をパッチに追加/削除`,
-		ToggleStagingPanel:      `パネルを切り替え`,
+		ToggleStagingView:       `パネルを切り替え`,
 		ReturnToFilesPanel:      `ファイル一覧に戻る`,
 		// FastForward:                         `fast-forward this branch from its upstream`,
 		// Fetching:                            "Fetching and fast-forwarding {{.from}} -> {{.to}} ...",
@@ -207,29 +200,31 @@ func japaneseTranslationSet() TranslationSet {
 		// NoRoom:                              "Not enough room",
 		YouAreHere: "現在位置",
 		// LcRewordNotSupported:                "Rewording commits while interactively rebasing is not currently supported",
-		CherryPickCopy:      "コミットをコピー (cherry-pick)",
-		CherryPickCopyRange: "コミットを範囲コピー (cherry-pick)",
-		PasteCommits:        "コミットを貼り付け (cherry-pick)",
+		CherryPickCopy: "コミットをコピー (cherry-pick)",
+		PasteCommits:   "コミットを貼り付け (cherry-pick)",
 		// SureCherryPick:                      "Are you sure you want to cherry-pick the copied commits onto this branch?",
-		CherryPick:          "Cherry-Pick",
-		Donate:              "支援",
-		AskQuestion:         "質問",
-		PrevLine:            "前の行を選択",
-		NextLine:            "次の行を選択",
-		PrevHunk:            "前のhunkを選択",
-		NextHunk:            "次のhunkを選択",
-		PrevConflict:        "前のコンフリクトを選択",
-		NextConflict:        "次のコンフリクトを選択",
-		SelectPrevHunk:      "前のhunkを選択",
-		SelectNextHunk:      "次のhunkを選択",
-		ScrollDown:          "下にスクロール",
-		ScrollUp:            "上にスクロール",
-		ScrollUpMainPanel:   "メインパネルを上にスクロール",
-		ScrollDownMainPanel: "メインパネルを下にスクロール",
-		AmendCommitTitle:    "Amendコミット",
-		AmendCommitPrompt:   "ステージされたファイルで現在のコミットをamendします。よろしいですか?",
-		DeleteCommitTitle:   "コミットを削除",
-		DeleteCommitPrompt:  "選択されたコミットを削除します。よろしいですか?",
+		CherryPick:           "Cherry-Pick",
+		Donate:               "支援",
+		AskQuestion:          "質問",
+		PrevLine:             "前の行を選択",
+		NextLine:             "次の行を選択",
+		PrevHunk:             "前のhunkを選択",
+		NextHunk:             "次のhunkを選択",
+		PrevConflict:         "前のコンフリクトを選択",
+		NextConflict:         "次のコンフリクトを選択",
+		SelectPrevHunk:       "前のhunkを選択",
+		SelectNextHunk:       "次のhunkを選択",
+		ScrollDown:           "下にスクロール",
+		ScrollUp:             "上にスクロール",
+		ScrollUpMainWindow:   "メインパネルを上にスクロール",
+		ScrollDownMainWindow: "メインパネルを下にスクロール",
+		AmendCommitTitle:     "Amendコミット",
+		AmendCommitPrompt:    "ステージされたファイルで現在のコミットをamendします。よろしいですか?",
+		DropCommitTitle:      "コミットを削除",
+		DropCommitPrompt:     "選択されたコミットを削除します。よろしいですか?",
+		PullingStatus:        "Pull中",
+		PushingStatus:        "Push中",
+		FetchingStatus:       "Fetch中",
 		// SquashingStatus:                     "Squashing",
 		// FixingStatus:                        "Fixing up",
 		// DeletingStatus:                      "Deleting",
@@ -265,12 +260,10 @@ func japaneseTranslationSet() TranslationSet {
 		// LcDiscardUntrackedFiles:             "Discard untracked files",
 		HardReset: "hardリセット",
 		// LcViewResetOptions:                  `view reset options`,
-		CreateFixupCommitDescription: `このコミットに対するfixupコミットを作成`,
+		CreateFixupCommitTooltip: `このコミットに対するfixupコミットを作成`,
 		// LcSquashAboveCommits:                `squash all 'fixup!' commits above selected commit (autosquash)`,
 		// SquashAboveCommits:                  `Squash all 'fixup!' commits above selected commit (autosquash)`,
-		SureSquashAboveCommits:   `{{.commit}}に対するすべての fixup! コミットをsquashします。よろしいですか?`,
 		CreateFixupCommit:        `Fixupコミットを作成`,
-		SureCreateFixupCommit:    `{{.commit}} に対する fixup! コミットを作成します。よろしいですか?`,
 		ExecuteCustomCommand:     "カスタムコマンドを実行",
 		CustomCommand:            "カスタムコマンド:",
 		CommitChangesWithoutHook: "pre-commitフックを実行せずに変更をコミット",
@@ -300,7 +293,7 @@ func japaneseTranslationSet() TranslationSet {
 		EnterUpstream:             `'<remote> <branchname>' の形式でupstreamを入力`,
 		InvalidUpstream:           "Upstreamの形式が正しくありません。'<remote> <branchname>' の形式で入力してください。",
 		ReturnToRemotesList:       `リモート一覧に戻る`,
-		AddNewRemote:              `リモートを新規追加`,
+		NewRemote:                 `リモートを新規追加`,
 		NewRemoteName:             `新規リモート名:`,
 		NewRemoteUrl:              `新規リモートURL:`,
 		EditRemoteName:            `{{.remoteName}} の新しいリモート名を入力:`,
@@ -312,20 +305,17 @@ func japaneseTranslationSet() TranslationSet {
 		// LcSetUpstream:                       "Set as upstream of checked-out branch",
 		// SetUpstreamTitle:                    "Set upstream branch",
 		// SetUpstreamMessage:                  "Are you sure you want to set the upstream branch of '{{.checkedOut}}' to '{{.selected}}'",
-		EditRemote:             "リモートを編集",
+		EditRemoteTooltip:      "リモートを編集",
 		TagCommit:              "タグを作成",
 		TagMenuTitle:           "タグを作成",
 		TagNameTitle:           "タグ名",
 		TagMessageTitle:        "タグメッセージ",
 		AnnotatedTag:           "注釈付きタグ",
 		LightweightTag:         "軽量タグ",
-		DeleteTag:              "タグを削除",
-		DeleteTagTitle:         "タグを削除",
-		DeleteTagPrompt:        "タグ '{{.tagName}}' を削除します。よろしいですか?",
 		PushTagTitle:           "リモートにタグ '{{.tagName}}' をpush",
 		PushTag:                "タグをpush",
-		CreateTag:              "タグを作成",
-		FetchRemote:            "リモートをfetch",
+		NewTag:                 "タグを作成",
+		FetchRemoteTooltip:     "リモートをfetch",
 		FetchingRemoteStatus:   "リモートをfetch",
 		CheckoutCommit:         "コミットをチェックアウト",
 		SureCheckoutThisCommit: "選択されたコミットをチェックアウトします。よろしいですか?",
@@ -343,7 +333,7 @@ func japaneseTranslationSet() TranslationSet {
 		RenameBranch:        "ブランチ名を変更",
 		NewBranchNamePrompt: "新しいブランチ名を入力",
 		// RenameBranchWarning:                 "This branch is tracking a remote. This action will only rename the local branch name, not the name of the remote branch. Continue?",
-		OpenMenu: "メニューを開く",
+		OpenKeybindingsMenu: "メニューを開く",
 		// LcResetCherryPick:                   "Reset cherry-picked (copied) commits selection",
 		NextTab:               "次のタブ",
 		PrevTab:               "前のタブ",
@@ -372,9 +362,9 @@ func japaneseTranslationSet() TranslationSet {
 		ExitDiffMode:     "差分モードを終了",
 		DiffingMenuTitle: "差分",
 		// LcSwapDiff:                          "Reverse diff direction",
-		OpenDiffingMenu: "差分メニューを開く",
+		ViewDiffingOptions: "差分メニューを開く",
 		// // the actual view is the extras view which I intend to give more tabs in future but for now we'll only mention the command log part
-		OpenExtrasMenu: "コマンドログメニューを開く",
+		OpenCommandLogMenu: "コマンドログメニューを開く",
 		// LcShowingGitDiff:                    "Showing output for:",
 		CommitDiff:                     "コミットの差分",
 		CopyCommitShaToClipboard:       "コミットのSHAをクリップボードにコピー",
@@ -385,9 +375,8 @@ func japaneseTranslationSet() TranslationSet {
 		CommitAuthor:                   "コミットの作成者名",
 		CopyCommitAttributeToClipboard: "コミットの情報をコピー",
 		CopyBranchNameToClipboard:      "ブランチ名をクリップボードにコピー",
-		CopyFileNameToClipboard:        "ファイル名をクリップボードにコピー",
-		CopyCommitFileNameToClipboard:  "コミットされたファイル名をクリップボードにコピー",
-		CopySelectedTexToClipboard:     "選択されたテキストをクリップボードにコピー",
+		CopyPathToClipboard:            "ファイル名をクリップボードにコピー",
+		CopySelectedTextToClipboard:    "選択されたテキストをクリップボードにコピー",
 		// LcCommitPrefixPatternError:          "Error in commitPrefix pattern",
 		NoFilesStagedTitle:         "ファイルがステージされていません",
 		NoFilesStagedPrompt:        "ファイルがステージされていません。すべての変更をコミットしますか?",
@@ -402,7 +391,7 @@ func japaneseTranslationSet() TranslationSet {
 		RunningCustomCommandStatus: "カスタムコマンドを実行",
 		// LcSubmoduleStashAndReset:            "Stash uncommitted submodule changes and update",
 		// LcAndResetSubmodules:                "And reset submodules",
-		EnterSubmodule:               "サブモジュールを開く",
+		EnterSubmoduleTooltip:        "サブモジュールを開く",
 		CopySubmoduleNameToClipboard: "サブモジュール名をクリップボードにコピー",
 		RemoveSubmodule:              "サブモジュールを削除",
 		RemoveSubmodulePrompt:        "サブモジュール '%s' とそのディレクトリを削除します。よろしいですか? この操作は取り消せません。",
@@ -410,14 +399,14 @@ func japaneseTranslationSet() TranslationSet {
 		NewSubmoduleName:             "新規サブモジュール名:",
 		NewSubmoduleUrl:              "新規サブモジュールのURL:",
 		NewSubmodulePath:             "新規サブモジュールのパス:",
-		AddSubmodule:                 "サブモジュールを新規追加",
+		NewSubmodule:                 "サブモジュールを新規追加",
 		AddingSubmoduleStatus:        "サブモジュールを新規追加",
 		UpdateSubmoduleUrl:           "サブモジュール '%s' のURLを更新",
 		UpdatingSubmoduleUrlStatus:   "URLを更新",
 		EditSubmoduleUrl:             "サブモジュールのURLを更新",
 		InitializingSubmoduleStatus:  "サブモジュールを初期化",
-		InitSubmodule:                "サブモジュールを初期化",
-		SubmoduleUpdate:              "サブモジュールを更新",
+		InitSubmoduleTooltip:         "サブモジュールを初期化",
+		SubmoduleUpdateTooltip:       "サブモジュールを更新",
 		UpdatingSubmoduleStatus:      "サブモジュールを更新",
 		BulkInitSubmodules:           "サブモジュールを一括初期化",
 		BulkUpdateSubmodules:         "サブモジュールを一括更新",
@@ -434,7 +423,6 @@ func japaneseTranslationSet() TranslationSet {
 		// PushingTagStatus:                    "Pushing tag",
 		PullRequestURLCopiedToClipboard:     "Pull requestのURLがクリップボードにコピーされました",
 		CommitDiffCopiedToClipboard:         "コミットの差分がクリップボードにコピーされました",
-		CommitSHACopiedToClipboard:          "コミットのSHAがクリップボードにコピーされました",
 		CommitURLCopiedToClipboard:          "コミットのURLがクリップボードにコピーされました",
 		CommitMessageCopiedToClipboard:      "コミットメッセージがクリップボードにコピーされました",
 		CommitAuthorCopiedToClipboard:       "コミットの作成者名がクリップボードにコピーされました",
@@ -467,8 +455,11 @@ func japaneseTranslationSet() TranslationSet {
 		OpenLogMenu:  "ログメニューを開く",
 		LogMenuTitle: "コミットログオプション",
 		// ToggleShowGitGraphAll:               "Toggle show whole git graph (pass the `--all` flag to `git log`)",
-		ShowGitGraph: "コミットグラフの表示",
-		SortCommits:  "コミットの表示順",
+		ShowGitGraph:     "コミットグラフの表示",
+		SortOrder:        "並び替え",
+		SortAlphabetical: "アルファベット順",
+		SortByDate:       "日付順",
+		SortCommits:      "コミットの表示順",
 		// CantChangeContextSizeError:          "Cannot change context while in patch building mode because we were too lazy to support it when releasing the feature. If you really want it, please let us know!",
 		OpenCommitInBrowser: "ブラウザでコミットを開く",
 		// LcViewBisectOptions:                 "View bisect options",
@@ -554,7 +545,6 @@ func japaneseTranslationSet() TranslationSet {
 			BulkUpdateSubmodules:              "サブモジュールを一括更新",
 			// BulkDeinitialiseSubmodules:        "Bulk deinitialise submodules",
 			UpdateSubmodule: "サブモジュールを更新",
-			DeleteTag:       "タグを削除",
 			PushTag:         "タグをpush",
 			// NukeWorkingTree:                   "Nuke working tree",
 			// DiscardUnstagedFileChanges:        "Discard unstaged file changes",
