@@ -15,6 +15,7 @@ import (
 	"testing"
 
 	"github.com/creack/pty"
+	"github.com/jesseduffield/lazycore/pkg/utils"
 	"github.com/jesseduffield/lazygit/pkg/integration/components"
 	"github.com/jesseduffield/lazygit/pkg/integration/tests"
 	"github.com/stretchr/testify/assert"
@@ -27,13 +28,18 @@ func TestIntegration(t *testing.T) {
 
 	parallelTotal := tryConvert(os.Getenv("PARALLEL_TOTAL"), 1)
 	parallelIndex := tryConvert(os.Getenv("PARALLEL_INDEX"), 0)
+	raceDetector := os.Getenv("LAZYGIT_RACE_DETECTOR") != ""
+	// LAZYGIT_GOCOVERDIR is the directory where we write coverage files to. If this directory
+	// is defined, go binaries built with the -cover flag will write coverage files to
+	// to it.
+	codeCoverageDir := os.Getenv("LAZYGIT_GOCOVERDIR")
 	testNumber := 0
 
-	err := components.RunTests(
-		tests.GetTests(),
-		t.Logf,
-		runCmdHeadless,
-		func(test *components.IntegrationTest, f func() error) {
+	err := components.RunTests(components.RunTestArgs{
+		Tests:  tests.GetTests(utils.GetLazyRootDirectory()),
+		Logf:   t.Logf,
+		RunCmd: runCmdHeadless,
+		TestWrapper: func(test *components.IntegrationTest, f func() error) {
 			defer func() { testNumber += 1 }()
 			if testNumber%parallelTotal != parallelIndex {
 				return
@@ -51,16 +57,19 @@ func TestIntegration(t *testing.T) {
 				assert.NoError(t, err)
 			})
 		},
-		false,
-		0,
+		Sandbox:         false,
+		WaitForDebugger: false,
+		RaceDetector:    raceDetector,
+		CodeCoverageDir: codeCoverageDir,
+		InputDelay:      0,
 		// Allow two attempts at each test to get around flakiness
-		2,
-	)
+		MaxAttempts: 2,
+	})
 
 	assert.NoError(t, err)
 }
 
-func runCmdHeadless(cmd *exec.Cmd) error {
+func runCmdHeadless(cmd *exec.Cmd) (int, error) {
 	cmd.Env = append(
 		cmd.Env,
 		"HEADLESS=true",
@@ -78,15 +87,16 @@ func runCmdHeadless(cmd *exec.Cmd) error {
 	// running other commands in a pty.
 	f, err := pty.StartWithSize(cmd, &pty.Winsize{Rows: 300, Cols: 300})
 	if err != nil {
-		return err
+		return -1, err
 	}
 
 	_, _ = io.Copy(io.Discard, f)
 
 	if cmd.Wait() != nil {
+		_ = f.Close()
 		// return an error with the stderr output
-		return errors.New(stderr.String())
+		return cmd.Process.Pid, errors.New(stderr.String())
 	}
 
-	return f.Close()
+	return cmd.Process.Pid, f.Close()
 }
